'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.verifyAsset = exports.default = exports.InvalidAssetError = undefined;

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _AssetType = require('./AssetType');

var _BaseState2 = require('./BaseState');

var _BaseState3 = _interopRequireDefault(_BaseState2);

require('./Equatable');

var _Serializable = require('./Serializable');

var _common = require('./common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('./crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let InvalidAssetError = exports.InvalidAssetError = function (_Error) {
  (0, _inherits3.default)(InvalidAssetError, _Error);

  function InvalidAssetError() {
    (0, _classCallCheck3.default)(this, InvalidAssetError);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidAssetError.__proto__ || (0, _getPrototypeOf2.default)(InvalidAssetError)).apply(this, arguments));
  }

  return InvalidAssetError;
}(Error);

let Asset = function (_BaseState) {
  (0, _inherits3.default)(Asset, _BaseState);

  function Asset({
    version,
    hash,
    type,
    name,
    amount,
    available,
    precision,
    feeMode,
    fee,
    feeAddress,
    owner,
    admin,
    issuer,
    expiration,
    isFrozen
  }) {
    (0, _classCallCheck3.default)(this, Asset);

    // eslint-disable-next-line
    var _this2 = (0, _possibleConstructorReturn3.default)(this, (Asset.__proto__ || (0, _getPrototypeOf2.default)(Asset)).call(this, { version }));

    _this2.equals = _utils2.default.equals(Asset, other => _common2.default.uInt256Equal(_this2.hash, other.hash));
    _this2.serializeWire = (0, _Serializable.createSerializeWire)(_this2.serializeWireBase.bind(_this2));
    verifyAsset({ name, type, amount, precision });
    _this2.hash = hash;
    _this2.hashHex = _common2.default.uInt256ToHex(hash);
    _this2.type = type;
    _this2.name = name;
    _this2.amount = amount;
    _this2.available = available || _utils2.default.ZERO;
    _this2.precision = precision;
    _this2.feeMode = feeMode || 0;
    _this2.fee = fee || _utils2.default.ZERO;
    _this2.feeAddress = feeAddress || _common2.default.ZERO_UINT160;
    _this2.owner = owner;
    _this2.admin = admin;
    _this2.issuer = issuer;
    _this2.expiration = expiration;
    _this2.isFrozen = isFrozen || false;
    return _this2;
  }

  (0, _createClass3.default)(Asset, [{
    key: 'update',
    value: function update({
      available,
      expiration,
      isFrozen
    }) {
      return new Asset({
        hash: this.hash,
        type: this.type,
        name: this.name,
        amount: this.amount,
        precision: this.precision,
        fee: this.fee,
        feeAddress: this.feeAddress,
        owner: this.owner,
        admin: this.admin,
        issuer: this.issuer,
        available: available == null ? this.available : available,
        expiration: expiration == null ? this.expiration : expiration,
        isFrozen: isFrozen == null ? this.isFrozen : isFrozen
      });
    }
  }, {
    key: 'serializeWireBase',
    value: function serializeWireBase(writer) {
      writer.writeUInt8(this.version);
      writer.writeUInt256(this.hash);
      writer.writeUInt8(this.type);
      writer.writeVarString(this.name);
      writer.writeFixed8(this.amount);
      writer.writeFixed8(this.available);
      writer.writeUInt8(this.precision);
      writer.writeUInt8(this.feeMode);
      writer.writeFixed8(this.fee);
      writer.writeUInt160(this.feeAddress);
      writer.writeECPoint(this.owner);
      writer.writeUInt160(this.admin);
      writer.writeUInt160(this.issuer);
      writer.writeUInt32LE(this.expiration);
      writer.writeBoolean(this.isFrozen);
    }
  }, {
    key: 'serializeJSON',


    // eslint-disable-next-line
    value: function serializeJSON(context) {
      let name = this.name || '';
      try {
        name = JSON.parse(name);
      } catch (error) {
        // eslint-disable-next-line
      }

      return {
        version: this.version,
        id: _utils.JSONHelper.writeUInt256(this.hash),
        type: (0, _AssetType.toJSONAssetType)(this.type),
        name,
        amount: _utils.JSONHelper.writeFixed8(this.amount),
        available: _utils.JSONHelper.writeFixed8(this.available),
        precision: this.precision,
        owner: _utils.JSONHelper.writeECPoint(this.owner),
        admin: _crypto2.default.scriptHashToAddress({
          addressVersion: context.addressVersion,
          scriptHash: this.admin
        }),
        issuer: _crypto2.default.scriptHashToAddress({
          addressVersion: context.addressVersion,
          scriptHash: this.issuer
        }),
        expiration: this.expiration,
        frozen: this.isFrozen
      };
    }
  }], [{
    key: 'deserializeWireBase',
    value: function deserializeWireBase({ reader }) {
      const version = reader.readUInt8();
      const hash = reader.readUInt256();
      const type = (0, _AssetType.assertAssetType)(reader.readUInt8());
      const name = reader.readVarString();
      const amount = reader.readFixed8();
      const available = reader.readFixed8();
      const precision = reader.readUInt8();
      reader.readUInt8(); // FeeMode
      const fee = reader.readFixed8();
      const feeAddress = reader.readUInt160();
      const owner = reader.readECPoint();
      const admin = reader.readUInt160();
      const issuer = reader.readUInt160();
      const expiration = reader.readUInt32LE();
      const isFrozen = reader.readBoolean();

      return new this({
        version,
        hash,
        type,
        name,
        amount,
        available,
        precision,
        fee,
        feeAddress,
        owner,
        admin,
        issuer,
        expiration,
        isFrozen
      });
    }
  }, {
    key: 'deserializeWire',
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);
  return Asset;
}(_BaseState3.default);

exports.default = Asset;
const verifyAsset = exports.verifyAsset = ({
  name,
  type,
  amount,
  precision
}) => {
  if (type === _AssetType.ASSET_TYPE.CREDIT_FLAG || type === _AssetType.ASSET_TYPE.DUTY_FLAG) {
    throw new InvalidAssetError('Asset type cannot be CREDIT_FLAG or DUTY_FLAG');
  }

  const nameBuffer = Buffer.from(name, 'utf8');
  if (nameBuffer.length > 1024) {
    throw new InvalidAssetError('Name too long');
  }

  if (amount.lte(_utils2.default.ZERO) && !amount.eq(_common2.default.NEGATIVE_SATOSHI_FIXED8)) {
    throw new InvalidAssetError('Amount must be greater than 0');
  }

  if (type === _AssetType.ASSET_TYPE.INVOICE && !amount.eq(_common2.default.NEGATIVE_SATOSHI_FIXED8)) {
    throw new InvalidAssetError('Invoice assets must have unlimited amount.');
  }

  if (precision > 8) {
    throw new InvalidAssetError('Max precision is 8.');
  }

  if (!amount.eq(_utils2.default.NEGATIVE_SATOSHI) && !amount.mod(_utils2.default.TEN.pow(_utils2.default.EIGHT.subn(precision))).eq(_utils2.default.ZERO)) {
    throw new InvalidAssetError('Invalid precision for amount.');
  }
};