'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BlockTransform = exports.InvalidBlockTransformEncodingError = exports.default = undefined;

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _stream = require('stream');

var _BlockBase2 = require('./BlockBase');

var _BlockBase3 = _interopRequireDefault(_BlockBase2);

require('./Serializable');

var _Header = require('./Header');

var _Header2 = _interopRequireDefault(_Header);

var _ScriptContainer = require('./ScriptContainer');

var _transaction = require('./transaction');

var _errors = require('./errors');

var _common = require('./common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('./crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let Block = function (_BlockBase) {
  (0, _inherits3.default)(Block, _BlockBase);

  function Block({
    version,
    previousHash,
    merkleRoot,
    timestamp,
    index,
    consensusData,
    nextConsensus,
    script,
    hash,
    transactions
  }) {
    (0, _classCallCheck3.default)(this, Block);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Block.__proto__ || (0, _getPrototypeOf2.default)(Block)).call(this, {
      version,
      previousHash,
      merkleRoot: merkleRoot || _crypto.MerkleTree.computeRoot(transactions.map(transaction => transaction.hash)),
      timestamp,
      index,
      consensusData,
      nextConsensus,
      script,
      hash
    }));

    _this._header = _utils2.default.lazy(() => new _Header2.default({
      version: _this.version,
      previousHash: _this.previousHash,
      merkleRoot: _this.merkleRoot,
      timestamp: _this.timestamp,
      index: _this.index,
      consensusData: _this.consensusData,
      nextConsensus: _this.nextConsensus,
      script: _this.script
    }));

    _this.transactions = transactions;
    _this.__blockSize = _utils2.default.lazy(() => (0, _get3.default)(Block.prototype.__proto__ || (0, _getPrototypeOf2.default)(Block.prototype), 'size', _this) + _utils.IOHelper.sizeOfArray(_this.transactions, transaction => transaction.size));
    return _this;
  }

  (0, _createClass3.default)(Block, [{
    key: 'getNetworkFee',
    value: (() => {
      var _ref = (0, _asyncToGenerator3.default)(function* (context) {
        const fees = yield _promise2.default.all(this.transactions.map(function (transaction) {
          return transaction.getNetworkFee(context);
        }));
        return fees.reduce(function (acc, fee) {
          return acc.add(fee);
        }, _utils2.default.ZERO);
      });

      function getNetworkFee(_x) {
        return _ref.apply(this, arguments);
      }

      return getNetworkFee;
    })()
  }, {
    key: 'getSystemFee',
    value: function getSystemFee(context) {
      return this.transactions.reduce((acc, transaction) => acc.add(transaction.getSystemFee(context)), _utils2.default.ZERO);
    }
  }, {
    key: 'verify',
    value: (() => {
      var _ref2 = (0, _asyncToGenerator3.default)(function* (options) {
        const { completely = false } = options;

        if (this.transactions.length === 0 || this.transactions[0].type !== _transaction.TRANSACTION_TYPE.MINER || this.transactions.slice(1).some(function (transaction) {
          return transaction.type === _transaction.TRANSACTION_TYPE.MINER;
        })) {
          throw new _errors.VerifyError('Invalid miner transaction in block.');
        }

        yield _promise2.default.all([this._verifyBase(options), completely ? this._verifyComplete(options) : _promise2.default.resolve()]);
      });

      function verify(_x2) {
        return _ref2.apply(this, arguments);
      }

      return verify;
    })()
  }, {
    key: '_verifyBase',
    value: (() => {
      var _ref3 = (0, _asyncToGenerator3.default)(function* ({
        genesisBlock,
        tryGetBlock,
        tryGetHeader,
        verifyScript
      }) {
        if (this.hash === genesisBlock.hash) {
          return;
        }

        const existingBlock = yield tryGetBlock({ hashOrIndex: this.hash });
        if (existingBlock != null) {
          return;
        }

        const previousHeader = yield tryGetHeader({
          hashOrIndex: this.previousHash
        });
        if (previousHeader == null) {
          throw new _errors.VerifyError('Previous header does not exist.');
        }

        if (previousHeader.index + 1 !== this.index) {
          throw new _errors.VerifyError('Previous index + 1 does not match index.');
        }

        if (previousHeader.timestamp >= this.timestamp) {
          throw new _errors.VerifyError('Previous timestamp is greater than block.');
        }

        yield verifyScript({
          scriptContainer: { type: _ScriptContainer.SCRIPT_CONTAINER_TYPE.BLOCK, value: this },
          hash: previousHeader.nextConsensus,
          witness: this.script
        });
      });

      function _verifyBase(_x3) {
        return _ref3.apply(this, arguments);
      }

      return _verifyBase;
    })()
  }, {
    key: '_verifyComplete',
    value: (() => {
      var _ref4 = (0, _asyncToGenerator3.default)(function* (options) {
        yield _promise2.default.all([this._verifyConsensus(options), this._verifyTransactions(options), this._verifyNetworkFee(options)]);
      });

      function _verifyComplete(_x4) {
        return _ref4.apply(this, arguments);
      }

      return _verifyComplete;
    })()
  }, {
    key: '_verifyConsensus',
    value: (() => {
      var _ref5 = (0, _asyncToGenerator3.default)(function* ({
        getValidators
      }) {
        const validators = yield getValidators(this.transactions);
        if (!_common2.default.uInt160Equal(this.nextConsensus, _crypto2.default.getConsensusAddress(validators))) {
          throw new _errors.VerifyError('Invalid next consensus address');
        }
      });

      function _verifyConsensus(_x5) {
        return _ref5.apply(this, arguments);
      }

      return _verifyConsensus;
    })()
  }, {
    key: '_verifyTransactions',
    value: (() => {
      var _ref6 = (0, _asyncToGenerator3.default)(function* (options) {
        yield _promise2.default.all(this.transactions.map(function (transaction) {
          return transaction.verify({
            isSpent: options.isSpent,
            getAsset: options.getAsset,
            getOutput: options.getOutput,
            calculateClaimAmount: options.calculateClaimAmount,
            verifyScript: options.verifyScript,
            currentHeight: options.currentHeight,
            governingToken: options.governingToken,
            utilityToken: options.utilityToken,
            fees: options.fees
          });
        }));
      });

      function _verifyTransactions(_x6) {
        return _ref6.apply(this, arguments);
      }

      return _verifyTransactions;
    })()
  }, {
    key: '_verifyNetworkFee',
    value: (() => {
      var _ref7 = (0, _asyncToGenerator3.default)(function* (options) {
        const networkFee = yield this.getNetworkFee({
          getOutput: options.getOutput,
          governingToken: options.governingToken,
          utilityToken: options.utilityToken,
          fees: options.fees
        });
        const minerTransaction = this.transactions.find(function (transaction) {
          return transaction.type === _transaction.TRANSACTION_TYPE.MINER;
        });
        if (minerTransaction == null) {
          throw new _errors.VerifyError('Missing miner transaction');
        }

        const minerTransactionNetworkFee = minerTransaction.outputs.reduce(function (acc, output) {
          return acc.add(output.value);
        }, _utils2.default.ZERO);

        if (!networkFee.eq(minerTransactionNetworkFee)) {
          throw new _errors.VerifyError('Miner output does not equal network fee.');
        }
      });

      function _verifyNetworkFee(_x7) {
        return _ref7.apply(this, arguments);
      }

      return _verifyNetworkFee;
    })()
  }, {
    key: 'serializeWireBase',
    value: function serializeWireBase(writer) {
      (0, _get3.default)(Block.prototype.__proto__ || (0, _getPrototypeOf2.default)(Block.prototype), 'serializeWireBase', this).call(this, writer);
      writer.writeArray(this.transactions, transaction => transaction.serializeWireBase(writer));
    }
  }, {
    key: 'serializeJSON',


    // eslint-disable-next-line
    value: (() => {
      var _ref8 = (0, _asyncToGenerator3.default)(function* (context) {
        const blockBaseJSON = (0, _get3.default)(Block.prototype.__proto__ || (0, _getPrototypeOf2.default)(Block.prototype), 'serializeBlockBaseJSON', this).call(this, context);

        return {
          version: blockBaseJSON.version,
          hash: blockBaseJSON.hash,
          previousblockhash: blockBaseJSON.previousblockhash,
          merkleroot: blockBaseJSON.merkleroot,
          time: blockBaseJSON.time,
          index: blockBaseJSON.index,
          nonce: blockBaseJSON.nonce,
          nextconsensus: blockBaseJSON.nextconsensus,
          script: blockBaseJSON.script,
          tx: yield _promise2.default.all(this.transactions.map(function (transaction) {
            return transaction.serializeJSON(context);
          })),
          size: blockBaseJSON.size,
          confirmations: blockBaseJSON.confirmations
        };
      });

      function serializeJSON(_x8) {
        return _ref8.apply(this, arguments);
      }

      return serializeJSON;
    })()
  }, {
    key: 'size',
    get: function () {
      return this.__blockSize();
    }
  }, {
    key: 'header',
    get: function () {
      return this._header();
    }
  }], [{
    key: 'deserializeWireBase',
    value: function deserializeWireBase(options) {
      const { reader } = options;
      const blockBase = (0, _get3.default)(Block.__proto__ || (0, _getPrototypeOf2.default)(Block), 'deserializeBlockBaseWireBase', this).call(this, options);
      const transactions = reader.readArray(() => (0, _transaction.deserializeWireBase)(options), 0x10000);
      if (transactions.length === 0) {
        throw new _errors.InvalidFormatError();
      }

      const merkleRoot = _crypto.MerkleTree.computeRoot(transactions.map(transaction => transaction.hash));
      if (!_common2.default.uInt256Equal(merkleRoot, blockBase.merkleRoot)) {
        throw new _errors.InvalidFormatError();
      }

      return new this({
        version: blockBase.version,
        previousHash: blockBase.previousHash,
        merkleRoot: blockBase.merkleRoot,
        timestamp: blockBase.timestamp,
        index: blockBase.index,
        consensusData: blockBase.consensusData,
        nextConsensus: blockBase.nextConsensus,
        script: blockBase.script,
        transactions
      });
    }
  }, {
    key: 'deserializeWire',
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);
  return Block;
}(_BlockBase3.default); /* @fl/*  */


exports.default = Block;

let InvalidBlockTransformEncodingError = exports.InvalidBlockTransformEncodingError = function (_Error) {
  (0, _inherits3.default)(InvalidBlockTransformEncodingError, _Error);

  function InvalidBlockTransformEncodingError() {
    (0, _classCallCheck3.default)(this, InvalidBlockTransformEncodingError);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidBlockTransformEncodingError.__proto__ || (0, _getPrototypeOf2.default)(InvalidBlockTransformEncodingError)).call(this, 'Invalid Block Transform Encoding.'));
  }

  return InvalidBlockTransformEncodingError;
}(Error);

const SIZE_OF_INT32 = 4;

let BlockTransform = exports.BlockTransform = function (_Transform) {
  (0, _inherits3.default)(BlockTransform, _Transform);

  function BlockTransform(context) {
    (0, _classCallCheck3.default)(this, BlockTransform);

    var _this3 = (0, _possibleConstructorReturn3.default)(this, (BlockTransform.__proto__ || (0, _getPrototypeOf2.default)(BlockTransform)).call(this, { readableObjectMode: true }));

    _this3.context = context;
    _this3.buffer = Buffer.from([]);
    return _this3;
  }

  (0, _createClass3.default)(BlockTransform, [{
    key: '_transform',
    value: function _transform(chunk, encoding, callback) {
      if (typeof chunk === 'string' || encoding !== 'buffer') {
        throw new InvalidBlockTransformEncodingError();
      }

      this.buffer = Buffer.concat([this.buffer, chunk]);
      try {
        const { remainingBuffer, blocks } = this._processBuffer(new _utils.BinaryReader(this.buffer));
        this.buffer = remainingBuffer;
        blocks.reverse().forEach(block => this.push(block));
        callback(null);
      } catch (error) {
        callback(error);
      }
    }
  }, {
    key: '_processBuffer',
    value: function _processBuffer(reader) {
      if (reader.remaining < SIZE_OF_INT32) {
        return { remainingBuffer: reader.remainingBuffer, blocks: [] };
      }

      const length = reader.clone().readInt32LE();

      // TODO: Not sure why this doesn't work properly with just length...
      if (reader.remaining + SIZE_OF_INT32 < length * 2) {
        return { remainingBuffer: reader.remainingBuffer, blocks: [] };
      }

      reader.readInt32LE();
      const block = Block.deserializeWireBase({
        context: this.context,
        reader
      });
      const { remainingBuffer, blocks } = this._processBuffer(reader);
      blocks.push(block);
      return { remainingBuffer, blocks };
    }
  }]);
  return BlockTransform;
}(_stream.Transform);