'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _Serializable = require('./Serializable');

require('./Equatable');

var _errors = require('./errors');

var _Witness = require('./Witness');

var _Witness2 = _interopRequireDefault(_Witness);

var _common = require('./common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('./crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let BlockBase = function () {
  function BlockBase({
    version,
    previousHash,
    merkleRoot,
    timestamp,
    index,
    consensusData,
    nextConsensus,
    script,
    hash
  }) {
    var _this = this;

    (0, _classCallCheck3.default)(this, BlockBase);
    this._hashHex = _utils2.default.lazy(() => _common2.default.uInt256ToHex(this.hash));
    this._message = _utils2.default.lazy(() => this.serializeUnsigned());
    this.__thisSize = _utils2.default.lazy(() => _utils.IOHelper.sizeOfUInt32LE + _utils.IOHelper.sizeOfUInt256 + _utils.IOHelper.sizeOfUInt256 + _utils.IOHelper.sizeOfUInt32LE + _utils.IOHelper.sizeOfUInt32LE + _utils.IOHelper.sizeOfUInt64LE + _utils.IOHelper.sizeOfUInt160 + _utils.IOHelper.sizeOfUInt8 + this.script.size);
    this.equals = _utils2.default.equals(this.constructor, other => _common2.default.uInt256Equal(this.hash, other.hash));
    this.getScriptHashesForVerifying = _utils2.default.lazyAsync((() => {
      var _ref = (0, _asyncToGenerator3.default)(function* ({
        getHeader
      }) {
        if (_this.index === 0) {
          return new _set2.default([_common2.default.uInt160ToHex(_crypto2.default.toScriptHash(_this.script.verification))]);
        }

        const previousHeader = yield getHeader({ hashOrIndex: _this.previousHash });
        return new _set2.default([_common2.default.uInt160ToHex(previousHeader.nextConsensus)]);
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })());
    this.serializeUnsigned = (0, _Serializable.createSerializeWire)(this.serializeUnsignedBase.bind(this));
    this.serializeWire = (0, _Serializable.createSerializeWire)(this.serializeWireBase.bind(this));

    this.version = version || 0;
    this.previousHash = previousHash;
    this.merkleRoot = merkleRoot;
    this.timestamp = timestamp;
    this.index = index;
    this.consensusData = consensusData;
    this.nextConsensus = nextConsensus;
    this.script = script;
    const hashIn = hash;
    this._hash = hashIn == null ? _utils2.default.lazy(() => _crypto2.default.hash256(this.message)) : () => hashIn;
  }

  (0, _createClass3.default)(BlockBase, [{
    key: 'serializeUnsignedBase',
    value: function serializeUnsignedBase(writer) {
      writer.writeUInt32LE(this.version);
      writer.writeUInt256(this.previousHash);
      writer.writeUInt256(this.merkleRoot);
      writer.writeUInt32LE(this.timestamp);
      writer.writeUInt32LE(this.index);
      writer.writeUInt64LE(this.consensusData);
      writer.writeUInt160(this.nextConsensus);
    }
  }, {
    key: 'serializeWireBase',
    value: function serializeWireBase(writer) {
      this.serializeUnsignedBase(writer);
      writer.writeUInt8(1);
      this.script.serializeWireBase(writer);
    }
  }, {
    key: 'serializeBlockBaseJSON',
    value: function serializeBlockBaseJSON(context) {
      return {
        version: this.version,
        hash: _utils.JSONHelper.writeUInt256(this.hash),
        size: this.size,
        previousblockhash: _utils.JSONHelper.writeUInt256(this.previousHash),
        merkleroot: _utils.JSONHelper.writeUInt256(this.merkleRoot),
        time: this.timestamp,
        index: this.index,
        nonce: _utils.JSONHelper.writeUInt64LE(this.consensusData),
        nextconsensus: _crypto2.default.scriptHashToAddress({
          addressVersion: context.addressVersion,
          scriptHash: this.nextConsensus
        }),
        script: this.script.serializeJSON(context),
        // TODO: Not sure if we should bother with this...
        confirmations: 0
      };
    }
  }, {
    key: 'hash',
    get: function () {
      return this._hash();
    }
  }, {
    key: 'hashHex',
    get: function () {
      return this._hashHex();
    }
  }, {
    key: 'message',
    get: function () {
      return this._message();
    }
  }, {
    key: 'size',
    get: function () {
      return this.__thisSize();
    }
  }], [{
    key: 'deserializeBlockBaseWireBase',
    value: function deserializeBlockBaseWireBase(options) {
      const { reader } = options;

      const version = reader.readUInt32LE();
      const previousHash = reader.readUInt256();
      const merkleRoot = reader.readUInt256();
      const timestamp = reader.readUInt32LE();
      const index = reader.readUInt32LE();
      const consensusData = reader.readUInt64LE();
      const nextConsensus = reader.readUInt160();
      if (reader.readUInt8() !== 1) {
        throw new _errors.InvalidFormatError();
      }
      const script = _Witness2.default.deserializeWireBase(options);

      return {
        version,
        previousHash,
        merkleRoot,
        timestamp,
        index,
        consensusData,
        nextConsensus,
        script
      };
    }
  }]);
  return BlockBase;
}();

exports.default = BlockBase;