'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.serializeContractWireBase = exports.deserializeContractWireBase = exports.sizeOfContract = exports.default = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _BaseState2 = require('./BaseState');

var _BaseState3 = _interopRequireDefault(_BaseState2);

var _contractParameter = require('./contractParameter');

require('./Equatable');

var _Serializable = require('./Serializable');

var _common = require('./common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('./crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let Contract = function (_BaseState) {
  (0, _inherits3.default)(Contract, _BaseState);

  function Contract({
    version,
    script,
    parameterList,
    returnType,
    hasStorage,
    name,
    codeVersion,
    author,
    email,
    description
  }) {
    (0, _classCallCheck3.default)(this, Contract);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Contract.__proto__ || (0, _getPrototypeOf2.default)(Contract)).call(this, { version }));

    _this.__contractSize = _utils2.default.lazy(() => sizeOfContract({
      script: _this.script,
      parameterList: _this.parameterList,
      name: _this.name,
      codeVersion: _this.codeVersion,
      author: _this.author,
      email: _this.email,
      description: _this.description
    }));
    _this._hash = _utils2.default.lazy(() => _crypto2.default.toScriptHash(_this.script));
    _this._hashHex = _utils2.default.lazy(() => _common2.default.uInt160ToHex(_this.hash));
    _this.equals = _utils2.default.equals(Contract, other => _common2.default.uInt160Equal(_this.hash, other.hash));
    _this.serializeWire = (0, _Serializable.createSerializeWire)(_this.serializeWireBase.bind(_this));

    _this.script = script;
    _this.parameterList = parameterList;
    _this.returnType = returnType;
    _this.hasStorage = hasStorage;
    _this.name = name;
    _this.codeVersion = codeVersion;
    _this.author = author;
    _this.email = email;
    _this.description = description;
    return _this;
  }

  // eslint-disable-next-line


  (0, _createClass3.default)(Contract, [{
    key: 'serializeWireBase',
    value: function serializeWireBase(writer) {
      // eslint-disable-next-line
      serializeContractWireBase({ writer, contract: this });
    }
  }, {
    key: 'serializeJSON',


    // eslint-disable-next-line
    value: function serializeJSON(context) {
      return {
        version: this.version,
        code: {
          hash: _utils.JSONHelper.writeUInt160(this.hash),
          script: _utils.JSONHelper.writeBuffer(this.script),
          parameters: this.parameterList.map(parameter => (0, _contractParameter.toJSONContractParameterType)(parameter)),
          returntype: (0, _contractParameter.toJSONContractParameterType)(this.returnType)
        },
        needstorage: this.hasStorage,
        name: this.name,
        code_version: this.codeVersion,
        author: this.author,
        email: this.email,
        description: this.description
      };
    }
  }, {
    key: 'size',
    get: function () {
      return this.__contractSize();
    }
  }, {
    key: 'hash',
    get: function () {
      return this._hash();
    }
  }, {
    key: 'hashHex',
    get: function () {
      return this._hashHex();
    }
  }], [{
    key: 'deserializeWireBase',
    value: function deserializeWireBase(options) {
      // eslint-disable-next-line
      return deserializeContractWireBase({
        context: options.context,
        reader: options.reader
      });
    }
  }, {
    key: 'deserializeWire',
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);
  return Contract;
}(_BaseState3.default);

exports.default = Contract;
const sizeOfContract = exports.sizeOfContract = ({
  script,
  parameterList,
  name,
  codeVersion,
  author,
  email,
  description,
  publishVersion
}) => _utils.IOHelper.sizeOfVarBytesLE(script) + _utils.IOHelper.sizeOfVarBytesLE(Buffer.from(parameterList)) + _utils.IOHelper.sizeOfUInt8 + (
// TODO: Doesn't take version into account?
publishVersion == null ? _utils.IOHelper.sizeOfBoolean : 0) + _utils.IOHelper.sizeOfVarString(name) + _utils.IOHelper.sizeOfVarString(codeVersion) + _utils.IOHelper.sizeOfVarString(author) + _utils.IOHelper.sizeOfVarString(email) + _utils.IOHelper.sizeOfVarString(description);

const deserializeContractWireBase = exports.deserializeContractWireBase = ({
  reader,
  publishVersion
}) => {
  const script = reader.readVarBytesLE();
  const parameterList = [...reader.readVarBytesLE()].map(value => (0, _contractParameter.assertContractParameterType)(value));
  const returnType = (0, _contractParameter.assertContractParameterType)(reader.readUInt8());
  const hasStorage = publishVersion == null || publishVersion >= 1 ? reader.readBoolean() : false;
  const name = reader.readVarString(252);
  const codeVersion = reader.readVarString(252);
  const author = reader.readVarString(252);
  const email = reader.readVarString(252);
  const description = reader.readVarString(65536);

  return new Contract({
    script,
    parameterList,
    returnType,
    hasStorage,
    name,
    codeVersion,
    author,
    email,
    description
  });
};

const serializeContractWireBase = exports.serializeContractWireBase = ({
  writer,
  contract,
  publishVersion
}) => {
  writer.writeVarBytesLE(contract.script);
  writer.writeVarBytesLE(Buffer.from(contract.parameterList));
  writer.writeUInt8(contract.returnType);
  if (publishVersion == null || publishVersion >= 1) {
    writer.writeBoolean(contract.hasStorage);
  }
  writer.writeVarString(contract.name);
  writer.writeVarString(contract.codeVersion);
  writer.writeVarString(contract.author);
  writer.writeVarString(contract.email);
  writer.writeVarString(contract.description);
};