'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _bignumber = require('bignumber.js');

var _bignumber2 = _interopRequireDefault(_bignumber);

var _errors = require('./errors');

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const add0x = value => `0x${value}`;
const strip0x = value => value.startsWith('0x') ? value.substring(2) : value;

// eslint-disable-next-line

// eslint-disable-next-line


const UINT160_BUFFER_BYTES = 20;

const bufferToUInt160 = value => {
  if (value.length !== UINT160_BUFFER_BYTES) {
    throw new _errors.InvalidFormatError();
  }

  return value;
};

const uInt160ToHex = value => add0x(_utils2.default.reverse(value).toString('hex'));

const hexToUInt160 = value => typeof value === 'string' ? bufferToUInt160(_utils2.default.reverse(Buffer.from(strip0x(value), 'hex'))) : value;

const uInt160ToBuffer = value => typeof value === 'string' ? hexToUInt160(value) : value;

const uInt160ToString = value => typeof value === 'string' ? value : uInt160ToHex(value);

const stringToUInt160 = value => hexToUInt160(value);

const uInt160Equal = (a, b) => a.equals(b);

const isUInt160 = value => value instanceof Buffer && value.length === UINT160_BUFFER_BYTES;

const asUInt160 = value => {
  if (value instanceof Buffer) {
    return bufferToUInt160(value);
  }

  throw new _errors.InvalidFormatError();
};

const ZERO_UINT160 = Buffer.alloc(20, 0);

// eslint-disable-next-line

// eslint-disable-next-line


const UINT256_BUFFER_BYTES = 32;
const ZERO_UINT256 = Buffer.alloc(UINT256_BUFFER_BYTES, 0);

const bufferToUInt256 = value => {
  if (value.length !== UINT256_BUFFER_BYTES) {
    throw new _errors.InvalidFormatError();
  }

  return value;
};

const uInt256ToHex = value => add0x(_utils2.default.reverse(value).toString('hex'));

const hexToUInt256 = value => typeof value === 'string' ? bufferToUInt256(_utils2.default.reverse(Buffer.from(strip0x(value), 'hex'))) : value;

const uInt256ToBuffer = value => typeof value === 'string' ? hexToUInt256(value) : value;

const uInt256ToString = value => typeof value === 'string' ? value : uInt256ToHex(value);

const stringToUInt256 = value => hexToUInt256(value);

const uInt256Equal = (a, b) => a.equals(b);

const toUInt32LE = bytes => new _bn2.default(uInt256ToBuffer(bytes).slice(0, 4), 'le').toNumber();

// eslint-disable-next-line

// eslint-disable-next-line


// Encoded compressed ECPoint
const ECPOINT_BUFFER_BYTES = 33;
const ECPOINT_INFINITY_BYTE = 0x00;
const ECPOINT_INFINITY = Buffer.from([ECPOINT_INFINITY_BYTE]);

const ecPointToHex = value => typeof value === 'string' ? value : value.toString('hex');

const hexToECPoint = value => typeof value === 'string' ? Buffer.from(value, 'hex') : value;

const ecPointToBuffer = value => typeof value === 'string' ? hexToECPoint(value) : value;

const bufferToECPoint = value => {
  if (value.length !== ECPOINT_BUFFER_BYTES) {
    if (value.length === 1 && value.equals(ECPOINT_INFINITY)) {
      return value;
    }

    throw new _errors.InvalidFormatError();
  }

  return value;
};

const ecPointToString = value => typeof value === 'string' ? value : ecPointToHex(value);

const stringToECPoint = value => hexToECPoint(value);

const ecPointEqual = (a, b) => a.equals(b);

const ecPointCompare = (a, b) => {
  const aHex = ecPointToHex(a);
  const bHex = ecPointToHex(b);
  if (aHex < bHex) {
    return -1;
  } else if (aHex > bHex) {
    return 1;
  }

  return 0;
};

const ecPointIsInfinity = value => value.equals(ECPOINT_INFINITY);

// eslint-disable-next-line

// eslint-disable-next-line


const PRIVATE_KEY_BUFFER_BYTES = 32;

const privateKeyToHex = value => typeof value === 'string' ? value : value.toString('hex');

const hexToPrivateKey = value => typeof value === 'string' ? Buffer.from(value, 'hex') : value;

const privateKeyToBuffer = value => typeof value === 'string' ? hexToECPoint(value) : value;

const bufferToPrivateKey = value => {
  if (value.length !== PRIVATE_KEY_BUFFER_BYTES) {
    throw new _errors.InvalidFormatError();
  }

  return value;
};

const privateKeyToString = value => typeof value === 'string' ? value : ecPointToHex(value);

const stringToPrivateKey = value => hexToECPoint(value);

const D = new _bn2.default(100000000);
const DBigNumber = new _bignumber2.default(D.toString(10));

const fixed8FromDecimal = value => {
  if (typeof value === 'number') {
    return new _bn2.default(value).mul(D);
  }

  if (value instanceof _bn2.default) {
    return value.mul(D);
  }

  let valueBigNumber = value;
  if (typeof value === 'string') {
    valueBigNumber = new _bignumber2.default(value);
  }

  // $FlowFixMe
  return new _bn2.default(valueBigNumber.times(DBigNumber).toString(), 10);
};

const fixed8ToDecimal = bn => new _bignumber2.default(bn.toString(10)).div(DBigNumber);

const NEGATIVE_SATOSHI_FIXED8 = new _bn2.default(-1);
const ONE_HUNDRED_FIXED8 = fixed8FromDecimal(100);

exports.default = {
  D,
  NEO_ADDRESS_VERSION: 23,
  NEO_PRIVATE_KEY_VERSION: 0x80,
  ECPOINT_BUFFER_BYTES,
  ECPOINT_INFINITY,
  ECPOINT_INFINITY_BYTE,
  PRIVATE_KEY_BUFFER_BYTES,
  UINT160_BUFFER_BYTES,
  UINT256_BUFFER_BYTES,
  ZERO_UINT160,
  ZERO_UINT256,
  NEGATIVE_SATOSHI_FIXED8,
  ONE_HUNDRED_FIXED8,
  uInt160ToBuffer,
  bufferToUInt160,
  uInt160ToHex,
  hexToUInt160,
  uInt160ToString,
  stringToUInt160,
  uInt160Equal,
  isUInt160,
  asUInt160,
  uInt256ToBuffer,
  bufferToUInt256,
  uInt256ToHex,
  hexToUInt256,
  uInt256ToString,
  stringToUInt256,
  uInt256Equal,
  toUInt32LE,
  ecPointToBuffer,
  bufferToECPoint,
  ecPointToHex,
  hexToECPoint,
  ecPointToString,
  ecPointCompare,
  stringToECPoint,
  ecPointEqual,
  ecPointIsInfinity,
  privateKeyToHex,
  hexToPrivateKey,
  privateKeyToBuffer,
  bufferToPrivateKey,
  privateKeyToString,
  stringToPrivateKey,
  fixed8FromDecimal,
  fixed8ToDecimal
};