'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toJSONContractParameterType = exports.assertContractParameterType = exports.InvalidContractParameterTypeError = exports.CONTRACT_PARAMETER_TYPE = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const CONTRACT_PARAMETER_TYPE = exports.CONTRACT_PARAMETER_TYPE = {
  SIGNATURE: 0x00,
  BOOLEAN: 0x01,
  INTEGER: 0x02,
  HASH160: 0x03,
  HASH256: 0x04,
  BYTE_ARRAY: 0x05,
  PUBLIC_KEY: 0x06,
  STRING: 0x07,
  ARRAY: 0x10,
  INTEROP_INTERFACE: 0xf0,
  VOID: 0xff
};

let InvalidContractParameterTypeError = exports.InvalidContractParameterTypeError = function (_Error) {
  (0, _inherits3.default)(InvalidContractParameterTypeError, _Error);

  function InvalidContractParameterTypeError(contractParameterType) {
    (0, _classCallCheck3.default)(this, InvalidContractParameterTypeError);

    var _this = (0, _possibleConstructorReturn3.default)(this, (InvalidContractParameterTypeError.__proto__ || (0, _getPrototypeOf2.default)(InvalidContractParameterTypeError)).call(this, `Expected contract parameter type, ` + `found: ${contractParameterType.toString(16)}`));

    _this.contractParameterType = contractParameterType;
    return _this;
  }

  return InvalidContractParameterTypeError;
}(Error);

const assertContractParameterType = exports.assertContractParameterType = value => {
  switch (value) {
    case CONTRACT_PARAMETER_TYPE.SIGNATURE:
      return CONTRACT_PARAMETER_TYPE.SIGNATURE;
    case CONTRACT_PARAMETER_TYPE.BOOLEAN:
      return CONTRACT_PARAMETER_TYPE.BOOLEAN;
    case CONTRACT_PARAMETER_TYPE.INTEGER:
      return CONTRACT_PARAMETER_TYPE.INTEGER;
    case CONTRACT_PARAMETER_TYPE.HASH160:
      return CONTRACT_PARAMETER_TYPE.HASH160;
    case CONTRACT_PARAMETER_TYPE.HASH256:
      return CONTRACT_PARAMETER_TYPE.HASH256;
    case CONTRACT_PARAMETER_TYPE.BYTE_ARRAY:
      return CONTRACT_PARAMETER_TYPE.BYTE_ARRAY;
    case CONTRACT_PARAMETER_TYPE.PUBLIC_KEY:
      return CONTRACT_PARAMETER_TYPE.PUBLIC_KEY;
    case CONTRACT_PARAMETER_TYPE.STRING:
      return CONTRACT_PARAMETER_TYPE.STRING;
    case CONTRACT_PARAMETER_TYPE.ARRAY:
      return CONTRACT_PARAMETER_TYPE.ARRAY;
    case CONTRACT_PARAMETER_TYPE.INTEROP_INTERFACE:
      return CONTRACT_PARAMETER_TYPE.INTEROP_INTERFACE;
    // TODO: Seems to be a bug in the TestNet
    case 0x16:
      return CONTRACT_PARAMETER_TYPE.ARRAY;
    case CONTRACT_PARAMETER_TYPE.VOID:
      return CONTRACT_PARAMETER_TYPE.VOID;
    default:
      throw new InvalidContractParameterTypeError(value);
  }
};

const toJSONContractParameterType = exports.toJSONContractParameterType = type => {
  switch (type) {
    case 0x00:
      return 'Signature';
    case 0x01:
      return 'Boolean';
    case 0x02:
      return 'Integer';
    case 0x03:
      return 'Hash160';
    case 0x04:
      return 'Hash256';
    case 0x05:
      return 'ByteArray';
    case 0x06:
      return 'PublicKey';
    case 0x07:
      return 'String';
    case 0x10:
      return 'Array';
    case 0xf0:
      return 'InteropInterface';
    case 0xff:
      return 'Void';
    default:
      // eslint-disable-next-line
      type;
      throw new InvalidContractParameterTypeError(type);
  }
};