'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('./crypto');

var _crypto2 = _interopRequireDefault(_crypto);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let InvalidMerkleTreeException = function (_Error) {
  (0, _inherits3.default)(InvalidMerkleTreeException, _Error);

  function InvalidMerkleTreeException() {
    (0, _classCallCheck3.default)(this, InvalidMerkleTreeException);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidMerkleTreeException.__proto__ || (0, _getPrototypeOf2.default)(InvalidMerkleTreeException)).call(this, 'Invalid Merkle tree.'));
  }

  return InvalidMerkleTreeException;
}(Error);

let MerkleTreeNode = function () {
  function MerkleTreeNode({
    hash,
    leftChild,
    rightChild,
    parent
  }) {
    (0, _classCallCheck3.default)(this, MerkleTreeNode);

    this.hash = hash;
    this.leftChild = leftChild;
    this.rightChild = rightChild;
    this.parent = parent;
  }

  (0, _createClass3.default)(MerkleTreeNode, [{
    key: 'clone',
    value: function clone(parent) {
      const self = new MerkleTreeNode({
        hash: this.hash,
        parent
      });
      if (this.leftChild != null) {
        self.leftChild = this.leftChild.clone(self);
      }
      if (this.rightChild != null) {
        self.rightChild = this.rightChild.clone(self);
      }
      return self;
    }
  }]);
  return MerkleTreeNode;
}();

const build = leavesIn => {
  const leaves = leavesIn;
  if (leaves.length === 0) {
    throw new InvalidMerkleTreeException();
  }
  if (leaves.length === 1) {
    return leaves[0];
  }

  const parents = [];
  const length = Math.floor((leaves.length + 1) / 2);
  for (let i = 0; i < length; i += 1) {
    const leftChild = leaves[i * 2];

    let rightChild;
    if (i * 2 + 1 === leaves.length) {
      rightChild = leftChild;
    } else {
      rightChild = leaves[i * 2 + 1];
    }
    const node = new MerkleTreeNode({
      hash: _crypto2.default.hash256(Buffer.concat([_common2.default.uInt256ToBuffer(leftChild.hash), _common2.default.uInt256ToBuffer(rightChild.hash)])),
      leftChild,
      rightChild
    });
    parents[i] = node;
    leaves[i * 2].parent = node;

    if (i * 2 + 1 !== leaves.length) {
      leaves[i * 2 + 1].parent = node;
    }
  }

  return build(parents);
};

const _trim = (node, index, depth, flags) => {
  if (depth === 1 || node.leftChild == null) {
    return;
  }

  if (depth === 2) {
    if (!flags[index * 2] && !flags[index * 2 + 1]) {
      // eslint-disable-next-line
      node.leftChild = null;
      // eslint-disable-next-line
      node.rightChild = null;
    }
  } else {
    const leftChild = node.leftChild;
    const rightChild = node.rightChild;
    if (leftChild != null && rightChild != null) {
      _trim(leftChild, index * 2, depth - 1, flags);
      _trim(rightChild, index * 2 + 1, depth - 1, flags);
      if (node.leftChild != null && node.rightChild != null && node.leftChild.leftChild == null && node.rightChild.rightChild == null) {
        // eslint-disable-next-line
        node.leftChild = null;
        // eslint-disable-next-line
        node.rightChild = null;
      }
    }
  }
};

const _depthFirstSearch = (node, hashes) => {
  const leftChild = node.leftChild;
  const rightChild = node.rightChild;
  if (leftChild == null || rightChild == null) {
    hashes.push(node.hash);
  } else {
    _depthFirstSearch(leftChild, hashes);
    _depthFirstSearch(rightChild, hashes);
  }
};

const _depthFirstSearch2 = node => {
  const hashes = [];
  _depthFirstSearch(node, hashes);
  return hashes;
};

let MerkleTree = function () {
  function MerkleTree(hashesOrNode) {
    (0, _classCallCheck3.default)(this, MerkleTree);

    this.root = Array.isArray(hashesOrNode) ? build(hashesOrNode.map(hash => new MerkleTreeNode({ hash }))) : hashesOrNode;
    this.depth = 1;
    for (let node = this.root; node.leftChild != null; node = node.leftChild) {
      this.depth += 1;
    }
  }

  (0, _createClass3.default)(MerkleTree, [{
    key: 'trim',
    value: function trim(flags) {
      const result = this.root.clone();
      _trim(result, 0, this.depth, flags);
      return new MerkleTree(result);
    }
  }, {
    key: 'depthFirstSearch',
    value: function depthFirstSearch() {
      return _depthFirstSearch2(this.root);
    }
  }, {
    key: 'toHashArray',
    value: function toHashArray() {
      return this.depthFirstSearch();
    }
  }], [{
    key: 'computeRoot',
    value: function computeRoot(hashes) {
      const tree = new this(hashes);
      return tree.root.hash;
    }
  }]);
  return MerkleTree;
}();

exports.default = MerkleTree;