'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.InvalidAddressError = exports.Base58CheckError = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _cryptoJs = require('crypto-js');

var _cryptoJs2 = _interopRequireDefault(_cryptoJs);

var _wif = require('wif');

var _wif2 = _interopRequireDefault(_wif);

var _bs = require('bs58');

var _bs2 = _interopRequireDefault(_bs);

var _elliptic = require('elliptic');

var _errors = require('../errors');

var _utils = require('../utils');

var _Witness = require('../Witness');

var _Witness2 = _interopRequireDefault(_Witness);

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ec = new _elliptic.ec('p256');

let Base58CheckError = exports.Base58CheckError = function (_Error) {
  (0, _inherits3.default)(Base58CheckError, _Error);

  function Base58CheckError() {
    (0, _classCallCheck3.default)(this, Base58CheckError);
    return (0, _possibleConstructorReturn3.default)(this, (Base58CheckError.__proto__ || (0, _getPrototypeOf2.default)(Base58CheckError)).call(this, 'Base58 Check Decode Error.'));
  }

  return Base58CheckError;
}(Error);

let InvalidAddressError = exports.InvalidAddressError = function (_Error2) {
  (0, _inherits3.default)(InvalidAddressError, _Error2);

  function InvalidAddressError(address) {
    (0, _classCallCheck3.default)(this, InvalidAddressError);

    var _this2 = (0, _possibleConstructorReturn3.default)(this, (InvalidAddressError.__proto__ || (0, _getPrototypeOf2.default)(InvalidAddressError)).call(this, `Invalid Address: ${address}`));

    _this2.address = address;
    return _this2;
  }

  return InvalidAddressError;
}(Error);

const toBuffer = wordArray => Buffer.from(wordArray.toString(_cryptoJs2.default.enc.Hex), 'hex');

const toWordArray = buffer => _cryptoJs2.default.enc.Hex.parse(buffer.toString('hex'));

const sha1 = value => toBuffer(_cryptoJs2.default.SHA1(toWordArray(value)));

const sha256 = value => toBuffer(_cryptoJs2.default.SHA256(toWordArray(value)));

const hash160 = value => _common2.default.bufferToUInt160(toBuffer(_cryptoJs2.default.RIPEMD160(_cryptoJs2.default.SHA256(toWordArray(value)))));

const hash256 = value => _common2.default.bufferToUInt256(toBuffer(_cryptoJs2.default.SHA256(_cryptoJs2.default.SHA256(toWordArray(value)))));

// TODO: This + verify should probably handle DER format signatures as well
const sign = ({
  message,
  privateKey
}) => {
  const sig = ec.sign(sha256(message), _common2.default.privateKeyToBuffer(privateKey));
  return Buffer.concat([sig.r.toArrayLike(Buffer, 'be', 32), sig.s.toArrayLike(Buffer, 'be', 32)]);
};

let InvalidSignatureError = function (_Error3) {
  (0, _inherits3.default)(InvalidSignatureError, _Error3);

  function InvalidSignatureError() {
    (0, _classCallCheck3.default)(this, InvalidSignatureError);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidSignatureError.__proto__ || (0, _getPrototypeOf2.default)(InvalidSignatureError)).call(this, 'Invalid Signature'));
  }

  return InvalidSignatureError;
}(Error);

const verify = ({
  message,
  signature,
  publicKey
}) => {
  if (signature.length !== 64) {
    throw new InvalidSignatureError();
  }

  const r = signature.slice(0, 32);
  const s = signature.slice(32);
  return ec.verify(sha256(message), { r, s }, publicKey);
};

let InvalidPrivateKeyError = function (_Error4) {
  (0, _inherits3.default)(InvalidPrivateKeyError, _Error4);

  function InvalidPrivateKeyError() {
    (0, _classCallCheck3.default)(this, InvalidPrivateKeyError);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidPrivateKeyError.__proto__ || (0, _getPrototypeOf2.default)(InvalidPrivateKeyError)).call(this, 'Invalid Private Key'));
  }

  return InvalidPrivateKeyError;
}(Error);

const toECPointFromKeyPair = pair => _common2.default.bufferToECPoint(Buffer.from(pair.getPublic(true, 'hex'), 'hex'));

const getPublicKey = privateKey => {
  const key = ec.keyFromPrivate(_common2.default.privateKeyToBuffer(privateKey));
  key.getPublic(true, 'hex');
  const { result } = key.validate();
  if (!result) {
    throw new InvalidPrivateKeyError();
  }

  return toECPointFromKeyPair(key);
};

const createKeyPair = () => {
  const key = ec.genKeyPair();

  return {
    privateKey: _common2.default.bufferToPrivateKey(key.getPrivate().toArrayLike(Buffer, 'be')),
    publicKey: toECPointFromKeyPair(key)
  };
};

const toScriptHash = value => hash160(value);

// Takes various formats and converts to standard ECPoint
const toECPoint = publicKey => toECPointFromKeyPair(ec.keyFromPublic(publicKey));

const isInfinity = ecPoint => ec.keyFromPublic(ecPoint).getPublic().isInfinity();

const base58Checksum = buffer => _common2.default.uInt256ToBuffer(hash256(buffer)).slice(0, 4);

const base58CheckEncode = buffer => {
  const checksum = base58Checksum(buffer);
  return _bs2.default.encode(Buffer.concat([buffer, checksum]));
};

const base58CheckDecode = value => {
  const buffer = _bs2.default.decode(value);
  const payload = buffer.slice(0, -4);
  const checksum = buffer.slice(-4);
  const payloadChecksum = base58Checksum(payload);
  if (!checksum.equals(payloadChecksum)) {
    throw new Base58CheckError();
  }

  return payload;
};

const scriptHashToAddress = ({
  addressVersion,
  scriptHash
}) => {
  const buffer = Buffer.allocUnsafe(21);
  buffer[0] = addressVersion;
  _common2.default.uInt160ToBuffer(scriptHash).copy(buffer, 1);
  return base58CheckEncode(buffer);
};

const addressToScriptHash = ({
  addressVersion,
  address
}) => {
  const decodedAddress = base58CheckDecode(address);
  if (decodedAddress.length !== 21 || decodedAddress[0] !== addressVersion) {
    throw new InvalidAddressError(address);
  }

  return _common2.default.bufferToUInt160(decodedAddress.slice(1));
};

const createInvocationScript = (message, privateKey) => {
  const builder = new _utils.ScriptBuilder();
  builder.emitPush(sign({ message, privateKey }));
  return builder.build();
};

const createVerificationScript = publicKey => {
  const builder = new _utils.ScriptBuilder();
  builder.emitPushECPoint(publicKey);
  builder.emitOp('CHECKSIG');
  return builder.build();
};

const createWitness = (message, privateKey) => {
  const verification = createVerificationScript(getPublicKey(privateKey));
  const invocation = createInvocationScript(message, privateKey);
  return new _Witness2.default({ verification, invocation });
};

const getVerificationScriptHash = publicKey => toScriptHash(createVerificationScript(publicKey));

const compareKeys = (a, b) => {
  const aPublic = a.getPublic();
  const bPublic = b.getPublic();
  const result = aPublic.getX().cmp(bPublic.getX());
  if (result !== 0) {
    return result;
  }

  return aPublic.getY().cmp(bPublic.getY());
};

const sortKeys = publicKeys => publicKeys.map(publicKey => ec.keyFromPublic(publicKey)).sort(compareKeys).map(keyPair => toECPointFromKeyPair(keyPair));

const createMultiSignatureRedeemScript = (m, publicKeys) => {
  if (!(m >= 1 && m <= publicKeys.length && publicKeys.length <= 1024)) {
    // TODO: Better error
    throw new Error();
  }

  const builder = new _utils.ScriptBuilder();
  builder.emitPushInt(Math.floor(m));
  const publicKeysSorted = sortKeys(publicKeys);
  for (const ecPoint of publicKeysSorted) {
    builder.emitPushECPoint(ecPoint);
  }
  builder.emitPushInt(publicKeysSorted.length);
  builder.emitOp('CHECKMULTISIG');
  return builder.build();
};

const getConsensusAddress = validators => toScriptHash(createMultiSignatureRedeemScript(validators.length - (validators.length - 1) / 3, validators));

const wifToPrivateKey = (wif, privateKeyVersion) => {
  const privateKeyDecoded = base58CheckDecode(wif);

  if (privateKeyDecoded.length !== 34 || privateKeyDecoded[0] !== privateKeyVersion || privateKeyDecoded[33] !== 0x01) {
    throw new _errors.InvalidFormatError();
  }

  return _common2.default.bufferToPrivateKey(privateKeyDecoded.slice(1, 33));
};

const privateKeyToWif = (privateKey, privateKeyVersion) => _wif2.default.encode(privateKeyVersion, _common2.default.privateKeyToBuffer(privateKey), true);

exports.default = {
  sha1,
  sha256,
  hash160,
  hash256,
  sign,
  verify,
  getPublicKey,
  toScriptHash,
  toECPoint,
  isInfinity,
  createKeyPair,
  scriptHashToAddress,
  addressToScriptHash,
  createInvocationScript,
  createVerificationScript,
  createWitness,
  getVerificationScriptHash,
  createMultiSignatureRedeemScript,
  getConsensusAddress,
  privateKeyToWif,
  wifToPrivateKey
};