'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _TransactionType = require('./TransactionType');

require('../Serializable');

var _TransactionBase2 = require('./TransactionBase');

var _TransactionBase3 = _interopRequireDefault(_TransactionBase2);

var _errors = require('../errors');

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let InvocationTransaction = function (_TransactionBase) {
  (0, _inherits3.default)(InvocationTransaction, _TransactionBase);

  function InvocationTransaction({
    version,
    attributes,
    inputs,
    outputs,
    scripts,
    hash,
    gas,
    script
  }) {
    (0, _classCallCheck3.default)(this, InvocationTransaction);

    var _this = (0, _possibleConstructorReturn3.default)(this, (InvocationTransaction.__proto__ || (0, _getPrototypeOf2.default)(InvocationTransaction)).call(this, {
      version,
      type: _TransactionType.TRANSACTION_TYPE.INVOCATION,
      attributes,
      inputs,
      outputs,
      scripts,
      hash
    }));

    _this.gas = gas;
    _this.script = script;

    if (_this.version > 1) {
      throw new _errors.InvalidFormatError();
    }

    if (_this.script.length === 0) {
      throw new _errors.InvalidFormatError();
    }

    if (_this.gas.lt(_utils2.default.ZERO)) {
      throw new _errors.InvalidFormatError();
    }

    _this.__size = _utils2.default.lazy(() => (0, _get3.default)(InvocationTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(InvocationTransaction.prototype), 'size', _this) + _utils.IOHelper.sizeOfUInt8 +
    // TODO: Doesn't seem to add size of GAS
    _utils.IOHelper.sizeOfVarBytesLE(_this.script));
    return _this;
  }

  (0, _createClass3.default)(InvocationTransaction, [{
    key: 'clone',
    value: function clone(scripts) {
      return new this.constructor({
        version: this.version,
        attributes: this.attributes,
        inputs: this.inputs,
        outputs: this.outputs,
        scripts,
        gas: this.gas,
        script: this.script
      });
    }
  }, {
    key: 'serializeExclusiveBase',
    value: function serializeExclusiveBase(writer) {
      writer.writeVarBytesLE(this.script);
      if (this.version >= 1) {
        writer.writeFixed8(this.gas);
      }
    }
  }, {
    key: 'serializeJSON',


    // eslint-disable-next-line
    value: (() => {
      var _ref = (0, _asyncToGenerator3.default)(function* (context) {
        const transactionBaseJSON = yield (0, _get3.default)(InvocationTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(InvocationTransaction.prototype), 'serializeTransactionBaseJSON', this).call(this, context);

        const { asset, contracts, actions, result } = yield context.getInvocationData(this);

        const json = {
          type: 'InvocationTransaction',
          txid: transactionBaseJSON.txid,
          size: transactionBaseJSON.size,
          version: transactionBaseJSON.version,
          attributes: transactionBaseJSON.attributes,
          vin: transactionBaseJSON.vin,
          vout: transactionBaseJSON.vout,
          scripts: transactionBaseJSON.scripts,
          sys_fee: transactionBaseJSON.sys_fee,
          net_fee: transactionBaseJSON.net_fee,
          script: _utils.JSONHelper.writeBuffer(this.script),
          gas: _utils.JSONHelper.writeFixed8(this.gas),
          result: result.serializeJSON(context),
          asset: asset == null ? undefined : asset.serializeJSON(context),
          contracts: contracts.map(function (contract) {
            return contract.serializeJSON(context);
          }),
          actions: actions.map(function (action) {
            return action.serializeJSON(context);
          })
        };
        if (asset == null) {
          delete json.asset;
        }

        return json;
      });

      function serializeJSON(_x) {
        return _ref.apply(this, arguments);
      }

      return serializeJSON;
    })()

    // eslint-disable-next-line

  }, {
    key: 'getSystemFee',
    value: function getSystemFee(context) {
      return this.gas;
    }

    // eslint-disable-next-line

  }, {
    key: 'verify',
    value: (() => {
      var _ref2 = (0, _asyncToGenerator3.default)(function* (options) {
        if (!this.gas.mod(_utils2.default.ONE_HUNDRED_MILLION).eq(_utils2.default.ZERO)) {
          throw new _errors.VerifyError('Invalid GAS amount');
        }

        yield (0, _get3.default)(InvocationTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(InvocationTransaction.prototype), 'verify', this).call(this, options);
      });

      function verify(_x2) {
        return _ref2.apply(this, arguments);
      }

      return verify;
    })()
  }, {
    key: 'size',
    get: function () {
      return this.__size();
    }
  }], [{
    key: 'deserializeWireBase',
    value: function deserializeWireBase(options) {
      const { reader } = options;

      const { type, version } = (0, _get3.default)(InvocationTransaction.__proto__ || (0, _getPrototypeOf2.default)(InvocationTransaction), 'deserializeTransactionBaseStartWireBase', this).call(this, options);
      if (type !== _TransactionType.TRANSACTION_TYPE.INVOCATION) {
        throw new _errors.InvalidFormatError();
      }

      const script = reader.readVarBytesLE(65536);
      if (script.length === 0) {
        throw new _errors.InvalidFormatError();
      }

      let gas = _utils2.default.ZERO;
      if (version >= 1) {
        gas = reader.readFixed8();
      }

      const { attributes, inputs, outputs, scripts } = (0, _get3.default)(InvocationTransaction.__proto__ || (0, _getPrototypeOf2.default)(InvocationTransaction), 'deserializeTransactionBaseEndWireBase', this).call(this, options);

      return new this({
        version,
        attributes,
        inputs,
        outputs,
        scripts,
        script,
        gas
      });
    }
  }]);
  return InvocationTransaction;
}(_TransactionBase3.default);

InvocationTransaction.VERSION = 1;
exports.default = InvocationTransaction;