'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _get2 = require('babel-runtime/helpers/get');

var _get3 = _interopRequireDefault(_get2);

var _AssetType = require('../AssetType');

var _TransactionType = require('./TransactionType');

require('../Serializable');

var _TransactionBase2 = require('./TransactionBase');

var _TransactionBase3 = _interopRequireDefault(_TransactionBase2);

var _errors = require('../errors');

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('../crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let RegisterTransaction = function (_TransactionBase) {
  (0, _inherits3.default)(RegisterTransaction, _TransactionBase);

  function RegisterTransaction({
    version,
    attributes,
    inputs,
    outputs,
    scripts,
    hash,
    asset
  }) {
    (0, _classCallCheck3.default)(this, RegisterTransaction);

    var _this = (0, _possibleConstructorReturn3.default)(this, (RegisterTransaction.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction)).call(this, {
      version,
      type: _TransactionType.TRANSACTION_TYPE.REGISTER,
      attributes,
      inputs,
      outputs,
      scripts,
      hash
    }));

    _this.asset = asset;

    if (_this.version !== 0) {
      throw new _errors.InvalidFormatError();
    }

    if (_common2.default.ecPointIsInfinity(asset.owner) && asset.type !== _AssetType.ASSET_TYPE.GOVERNING_TOKEN && asset.type !== _AssetType.ASSET_TYPE.UTILITY_TOKEN) {
      throw new _errors.InvalidFormatError();
    }

    _this.__size = _utils2.default.lazy(() => (0, _get3.default)(RegisterTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction.prototype), 'size', _this) + _utils.IOHelper.sizeOfUInt8 + _utils.IOHelper.sizeOfUInt8 + _utils.IOHelper.sizeOfVarString(_this.asset.name) + _utils.IOHelper.sizeOfFixed8 + _utils.IOHelper.sizeOfUInt8 + _utils.IOHelper.sizeOfECPoint(_this.asset.owner) + _utils.IOHelper.sizeOfUInt160);
    _this.__registerGetScriptHashesForVerifying = _utils2.default.lazyAsync((() => {
      var _ref = (0, _asyncToGenerator3.default)(function* (options) {
        const hashes = yield (0, _get3.default)(RegisterTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction.prototype), 'getScriptHashesForVerifying', _this).call(_this, options);
        const scriptHash = _common2.default.uInt160ToHex(_crypto2.default.getVerificationScriptHash(_this.asset.owner));
        return new _set2.default([...hashes, scriptHash]);
      });

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    })());
    return _this;
  }

  (0, _createClass3.default)(RegisterTransaction, [{
    key: 'clone',
    value: function clone(scripts) {
      return new this.constructor({
        version: this.version,
        attributes: this.attributes,
        inputs: this.inputs,
        outputs: this.outputs,
        scripts,
        hash: this.hash,
        asset: this.asset
      });
    }
  }, {
    key: 'serializeExclusiveBase',
    value: function serializeExclusiveBase(writer) {
      writer.writeUInt8(this.asset.type);
      writer.writeVarString(this.asset.name);
      writer.writeFixed8(this.asset.amount);
      writer.writeUInt8(this.asset.precision);
      writer.writeECPoint(this.asset.owner);
      writer.writeUInt160(this.asset.admin);
    }
  }, {
    key: 'serializeJSON',


    // eslint-disable-next-line
    value: (() => {
      var _ref2 = (0, _asyncToGenerator3.default)(function* (context) {
        const transactionBaseJSON = yield (0, _get3.default)(RegisterTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction.prototype), 'serializeTransactionBaseJSON', this).call(this, context);

        let name = this.asset.name;
        try {
          name = JSON.parse(name);
        } catch (error) {
          // eslint-disable-next-line
        }

        return {
          type: 'RegisterTransaction',
          txid: transactionBaseJSON.txid,
          size: transactionBaseJSON.size,
          version: transactionBaseJSON.version,
          attributes: transactionBaseJSON.attributes,
          vin: transactionBaseJSON.vin,
          vout: transactionBaseJSON.vout,
          scripts: transactionBaseJSON.scripts,
          sys_fee: transactionBaseJSON.sys_fee,
          net_fee: transactionBaseJSON.net_fee,
          asset: {
            type: (0, _AssetType.toJSONAssetType)(this.asset.type),
            name,
            amount: _utils.JSONHelper.writeFixed8(this.asset.amount),
            precision: this.asset.precision,
            owner: _utils.JSONHelper.writeECPoint(this.asset.owner),
            admin: _crypto2.default.scriptHashToAddress({
              addressVersion: context.addressVersion,
              scriptHash: this.asset.admin
            })
          }
        };
      });

      function serializeJSON(_x2) {
        return _ref2.apply(this, arguments);
      }

      return serializeJSON;
    })()
  }, {
    key: 'getSystemFee',
    value: function getSystemFee(context) {
      if (this.asset.type === _AssetType.ASSET_TYPE.GOVERNING_TOKEN || this.asset.type === _AssetType.ASSET_TYPE.UTILITY_TOKEN) {
        return _utils2.default.ZERO;
      }

      return (0, _get3.default)(RegisterTransaction.prototype.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction.prototype), 'getSystemFee', this).call(this, context);
    }
  }, {
    key: 'getScriptHashesForVerifying',
    value: (() => {
      var _ref3 = (0, _asyncToGenerator3.default)(function* (options) {
        return this.__registerGetScriptHashesForVerifying(options);
      });

      function getScriptHashesForVerifying(_x3) {
        return _ref3.apply(this, arguments);
      }

      return getScriptHashesForVerifying;
    })()

    // eslint-disable-next-line

  }, {
    key: 'verify',
    value: (() => {
      var _ref4 = (0, _asyncToGenerator3.default)(function* (options) {
        throw new _errors.VerifyError('Enrollment transactions are obsolete');
      });

      function verify(_x4) {
        return _ref4.apply(this, arguments);
      }

      return verify;
    })()
  }, {
    key: 'size',
    get: function () {
      return this.__size();
    }
  }], [{
    key: 'deserializeWireBase',
    value: function deserializeWireBase(options) {
      const { reader } = options;

      const { type, version } = (0, _get3.default)(RegisterTransaction.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction), 'deserializeTransactionBaseStartWireBase', this).call(this, options);
      if (type !== _TransactionType.TRANSACTION_TYPE.REGISTER) {
        throw new _errors.InvalidFormatError();
      }

      const assetType = (0, _AssetType.assertAssetType)(reader.readUInt8());
      const name = reader.readVarString(1024);
      const amount = reader.readFixed8();
      const precision = reader.readUInt8();
      const owner = reader.readECPoint();
      const admin = reader.readUInt160();

      const { attributes, inputs, outputs, scripts } = (0, _get3.default)(RegisterTransaction.__proto__ || (0, _getPrototypeOf2.default)(RegisterTransaction), 'deserializeTransactionBaseEndWireBase', this).call(this, options);

      return new this({
        version,
        attributes,
        inputs,
        outputs,
        scripts,
        asset: {
          type: assetType,
          name,
          amount,
          precision,
          owner,
          admin
        }
      });
    }
  }]);
  return RegisterTransaction;
}(_TransactionBase3.default);

exports.default = RegisterTransaction;