'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.MAX_TRANSACTION_ATTRIBUTES = undefined;

var _set = require('babel-runtime/core-js/set');

var _set2 = _interopRequireDefault(_set);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _attribute = require('./attribute');

var _AssetType = require('../AssetType');

var _ScriptContainer = require('../ScriptContainer');

var _TransactionType = require('./TransactionType');

require('../Equatable');

var _Serializable = require('../Serializable');

var _errors = require('../errors');

var _Input = require('./Input');

var _Input2 = _interopRequireDefault(_Input);

var _Output = require('./Output');

var _Output2 = _interopRequireDefault(_Output);

var _Witness = require('../Witness');

var _Witness2 = _interopRequireDefault(_Witness);

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _crypto = require('../crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _common3 = require('./common');

var _utils = require('../utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getUtilityValue = ({
  outputs,
  utilityToken
}) => outputs.filter(output => _common2.default.uInt256Equal(output.asset, utilityToken.hash)).reduce((acc, output) => acc.add(output.value), _utils2.default.ZERO);

const MAX_TRANSACTION_ATTRIBUTES = exports.MAX_TRANSACTION_ATTRIBUTES = 16;

let TransactionBase = function () {
  function TransactionBase({
    version,
    type,
    attributes,
    inputs,
    outputs,
    scripts,
    hash
  }) {
    (0, _classCallCheck3.default)(this, TransactionBase);

    _initialiseProps.call(this);

    this.version = version == null ? this.constructor.VERSION : version;
    this.type = type;
    this.attributes = attributes || [];
    this.inputs = inputs || [];
    this.outputs = outputs || [];
    this.scripts = scripts || [];
    const hashIn = hash;
    this._hash = hashIn == null ? _utils2.default.lazy(() => _crypto2.default.hash256(this.message)) : () => hashIn;

    if (this.attributes.length > MAX_TRANSACTION_ATTRIBUTES) {
      throw new _errors.InvalidFormatError();
    }
  }

  (0, _createClass3.default)(TransactionBase, [{
    key: 'clone',


    // eslint-disable-next-line
    value: function clone(scripts) {
      throw new Error('Not Implemented');
    }
  }, {
    key: 'sign',
    value: function sign(key) {
      return this.clone(this.scripts.concat([_crypto2.default.createWitness(this.serializeUnsigned(), key)]));
    }
  }, {
    key: 'serializeExclusiveBase',


    // eslint-disable-next-line
    value: function serializeExclusiveBase(writer) {}
  }, {
    key: 'serializeUnsignedBase',
    value: function serializeUnsignedBase(writer) {
      writer.writeUInt8(this.type);
      writer.writeUInt8(this.version);
      this.serializeExclusiveBase(writer);
      writer.writeArray(this.attributes, attribute => {
        attribute.serializeWireBase(writer);
      });
      writer.writeArray(this.inputs, input => {
        input.serializeWireBase(writer);
      });
      writer.writeArray(this.outputs, output => {
        output.serializeWireBase(writer);
      });
    }
  }, {
    key: 'serializeWireBase',
    value: function serializeWireBase(writer) {
      this.serializeUnsignedBase(writer);
      writer.writeArray(this.scripts, script => {
        script.serializeWireBase(writer);
      });
    }
  }, {
    key: 'serializeTransactionBaseJSON',
    value: (() => {
      var _ref = (0, _asyncToGenerator3.default)(function* (context) {
        const networkFee = yield this.getNetworkFee(context.feeContext);
        return {
          txid: _common2.default.uInt256ToString(this.hashHex),
          size: this.size,
          version: this.version,
          attributes: this.attributes.map(function (attribute) {
            return attribute.serializeJSON(context);
          }),
          vin: this.inputs.map(function (input) {
            return input.serializeJSON(context);
          }),
          vout: this.outputs.map(function (output, index) {
            return output.serializeJSON(context, index);
          }),
          scripts: this.scripts.map(function (script) {
            return script.serializeJSON(context);
          }),
          sys_fee: _utils.JSONHelper.writeFixed8(this.getSystemFee(context.feeContext)),
          net_fee: _utils.JSONHelper.writeFixed8(networkFee)
        };
      });

      function serializeTransactionBaseJSON(_x) {
        return _ref.apply(this, arguments);
      }

      return serializeTransactionBaseJSON;
    })()

    // eslint-disable-next-line

  }, {
    key: 'serializeJSON',
    value: (() => {
      var _ref2 = (0, _asyncToGenerator3.default)(function* (context) {
        throw new Error('Not Implemented');
      });

      function serializeJSON(_x2) {
        return _ref2.apply(this, arguments);
      }

      return serializeJSON;
    })()
  }, {
    key: 'getNetworkFee',
    value: (() => {
      var _ref3 = (0, _asyncToGenerator3.default)(function* (context) {
        return this._networkFee(context);
      });

      function getNetworkFee(_x3) {
        return _ref3.apply(this, arguments);
      }

      return getNetworkFee;
    })()
  }, {
    key: 'getSystemFee',
    value: function getSystemFee({ fees }) {
      return fees[this.type] || _utils2.default.ZERO;
    }
  }, {
    key: 'getReferences',
    value: function getReferences(options) {
      return this.__getReferences(options);
    }
  }, {
    key: 'getTransactionResults',
    value: function getTransactionResults(options) {
      return this.__getTransactionResults(options);
    }
  }, {
    key: 'getScriptHashesForVerifying',
    value: (() => {
      var _ref4 = (0, _asyncToGenerator3.default)(function* (options) {
        return this.__baseGetScriptHashesForVerifying(options);
      });

      function getScriptHashesForVerifying(_x4) {
        return _ref4.apply(this, arguments);
      }

      return getScriptHashesForVerifying;
    })()
  }, {
    key: 'verify',
    value: (() => {
      var _ref5 = (0, _asyncToGenerator3.default)(function* (options) {
        var _this = this;

        const { memPool = [] } = options;
        if ((0, _common3.hasDuplicateInputs)(this.inputs)) {
          throw new _errors.VerifyError('Duplicate inputs');
        }

        if (memPool.some(function (tx) {
          return !tx.equals(_this) && tx.inputs.some(function (input) {
            return _this.inputs.some(function (thisInput) {
              return input.equals(thisInput);
            });
          });
        })) {
          throw new _errors.VerifyError('Input already exists in mempool');
        }

        if (this.attributes.filter(function (attribute) {
          return attribute.usage === _attribute.ATTRIBUTE_USAGE.ECDH02 || attribute.usage === _attribute.ATTRIBUTE_USAGE.ECDH03;
        }).length > 1) {
          throw new _errors.VerifyError('Too many ECDH attributes.');
        }

        yield _promise2.default.all([this._verifyDoubleSpend(options), this._verifyOutputs(options), this._verifyTransactionResults(options), this._verifyScripts(options)]);
      });

      function verify(_x5) {
        return _ref5.apply(this, arguments);
      }

      return verify;
    })()
  }, {
    key: '_verifyDoubleSpend',
    value: (() => {
      var _ref6 = (0, _asyncToGenerator3.default)(function* ({
        isSpent
      }) {
        const isDoubleSpend = yield _utils.async.some(this.inputs, function (input) {
          return isSpent(input);
        });
        if (isDoubleSpend) {
          throw new _errors.VerifyError('Transaction is a double spend');
        }
      });

      function _verifyDoubleSpend(_x6) {
        return _ref6.apply(this, arguments);
      }

      return _verifyDoubleSpend;
    })()
  }, {
    key: '_verifyOutputs',
    value: (() => {
      var _ref7 = (0, _asyncToGenerator3.default)(function* ({
        getAsset,
        currentHeight
      }) {
        const outputsGrouped = _utils2.default.entries(_lodash2.default.groupBy(this.outputs, function (output) {
          return _common2.default.uInt256ToHex(output.asset);
        }));
        const hasInvalidOutput = yield _utils.async.some(outputsGrouped, (() => {
          var _ref8 = (0, _asyncToGenerator3.default)(function* ([assetHex, outputs]) {
            const asset = yield getAsset({ hash: _common2.default.hexToUInt256(assetHex) });
            if (asset.expiration <= currentHeight + 1 && asset.type !== _AssetType.ASSET_TYPE.GOVERNING_TOKEN && asset.type !== _AssetType.ASSET_TYPE.UTILITY_TOKEN) {
              return true;
            }

            return outputs.some(function (output) {
              return !output.value.mod(_utils2.default.TEN.pow(_utils2.default.EIGHT.subn(asset.precision))).eq(_utils2.default.ZERO);
            });
          });

          return function (_x8) {
            return _ref8.apply(this, arguments);
          };
        })());
        if (hasInvalidOutput) {
          throw new _errors.VerifyError('Transaction has invalid output');
        }
      });

      function _verifyOutputs(_x7) {
        return _ref7.apply(this, arguments);
      }

      return _verifyOutputs;
    })()
  }, {
    key: '_verifyTransactionResults',
    value: (() => {
      var _ref9 = (0, _asyncToGenerator3.default)(function* ({
        getOutput,
        utilityToken,
        governingToken,
        fees
      }) {
        const results = yield this.getTransactionResults({ getOutput });
        const resultsDestroy = _utils2.default.entries(results).filter(
        // eslint-disable-next-line
        function ([_, value]) {
          return value.gt(_utils2.default.ZERO);
        });
        if (resultsDestroy.length > 1 || resultsDestroy.length === 1 && !_common2.default.uInt256Equal(_common2.default.hexToUInt256(resultsDestroy[0][0]), utilityToken.hash)) {
          throw new _errors.VerifyError('Invalid destroyed output.');
        }

        const feeContext = {
          getOutput,
          governingToken,
          utilityToken,
          fees
        };
        const systemFee = this.getSystemFee(feeContext);
        if (systemFee.gt(_utils2.default.ZERO) && (resultsDestroy.length === 0 || resultsDestroy[0][1].lt(systemFee))) {
          throw new _errors.VerifyError('Not enough output value for system fee.');
        }

        const resultsIssue = _utils2.default.entries(results).filter(
        // eslint-disable-next-line
        function ([_, value]) {
          return value.lt(_utils2.default.ZERO);
        });

        switch (this.type) {
          case _TransactionType.TRANSACTION_TYPE.MINER:
          case _TransactionType.TRANSACTION_TYPE.CLAIM:
            // eslint-disable-next-line
            if (resultsIssue.some(function ([assetHex, _]) {
              return !_common2.default.uInt256Equal(_common2.default.hexToUInt256(assetHex), utilityToken.hash);
            })) {
              throw new _errors.VerifyError('Invalid miner/claim result');
            }
            break;
          case _TransactionType.TRANSACTION_TYPE.ISSUE:
            // eslint-disable-next-line
            if (resultsIssue.some(function ([assetHex, _]) {
              return _common2.default.uInt256Equal(_common2.default.hexToUInt256(assetHex), utilityToken.hash);
            })) {
              throw new _errors.VerifyError('Invalid issue result');
            }
            break;
          default:
            if (resultsIssue.length > 0) {
              throw new _errors.VerifyError('Invalid results.');
            }
            break;
        }
      });

      function _verifyTransactionResults(_x9) {
        return _ref9.apply(this, arguments);
      }

      return _verifyTransactionResults;
    })()
  }, {
    key: '_verifyScripts',
    value: (() => {
      var _ref10 = (0, _asyncToGenerator3.default)(function* ({
        getAsset,
        getOutput,
        verifyScript
      }) {
        var _this2 = this;

        const hashesSet = yield this.getScriptHashesForVerifying({
          getAsset,
          getOutput
        });

        if (hashesSet.size !== this.scripts.length) {
          throw new _errors.VerifyError('Invalid witnesses');
        }

        const hashes = [...hashesSet].sort().map(function (value) {
          return _common2.default.hexToUInt160(value);
        });
        yield _promise2.default.all(_lodash2.default.zip(hashes, this.scripts).map(function ([hash, witness]) {
          return verifyScript({
            scriptContainer: {
              type: _ScriptContainer.SCRIPT_CONTAINER_TYPE.TRANSACTION,
              value: _this2
            },
            hash,
            witness
          });
        }));
      });

      function _verifyScripts(_x10) {
        return _ref10.apply(this, arguments);
      }

      return _verifyScripts;
    })()
  }, {
    key: 'hash',
    get: function () {
      return this._hash();
    }
  }, {
    key: 'hashHex',
    get: function () {
      return this._hashHex();
    }
  }, {
    key: 'size',
    get: function () {
      return this._size();
    }
  }, {
    key: 'message',
    get: function () {
      return this._message();
    }
  }], [{
    key: 'deserializeTransactionBaseStartWireBase',
    value: function deserializeTransactionBaseStartWireBase({ reader }) {
      const type = (0, _TransactionType.assertTransactionType)(reader.readUInt8());
      const version = reader.readUInt8();

      return { type, version };
    }
  }, {
    key: 'deserializeTransactionBaseEndWireBase',
    value: function deserializeTransactionBaseEndWireBase(options) {
      const { reader } = options;
      const attributes = reader.readArray(() => (0, _attribute.deserializeWireBase)(options), MAX_TRANSACTION_ATTRIBUTES);
      const inputs = reader.readArray(() => _Input2.default.deserializeWireBase(options));
      const outputs = reader.readArray(() => _Output2.default.deserializeWireBase(options), _utils2.default.USHORT_MAX_NUMBER + 1);
      const scripts = reader.readArray(() => _Witness2.default.deserializeWireBase(options));

      return { attributes, inputs, outputs, scripts };
    }
  }, {
    key: 'deserializeWireBase',


    // eslint-disable-next-line
    value: function deserializeWireBase(options) {
      throw new Error('Not Implemented');
    }
  }, {
    key: 'deserializeWire',
    value: function deserializeWire(options) {
      return this.deserializeWireBase({
        context: options.context,
        reader: new _utils.BinaryReader(options.buffer)
      });
    }
  }]);
  return TransactionBase;
}();

TransactionBase.VERSION = 0;

var _initialiseProps = function () {
  var _this3 = this;

  this._hashHex = _utils2.default.lazy(() => _common2.default.uInt256ToHex(this.hash));
  this._size = _utils2.default.lazy(() => _utils.IOHelper.sizeOfUInt8 + _utils.IOHelper.sizeOfArray(this.attributes, attribute => attribute.size) + _utils.IOHelper.sizeOfArray(this.inputs, input => input.size) + _utils.IOHelper.sizeOfArray(this.outputs, output => output.size) + _utils.IOHelper.sizeOfArray(this.scripts, script => script.size));
  this._message = _utils2.default.lazy(() => this.serializeUnsigned());
  this.equals = _utils2.default.equals(this.constructor, other => _common2.default.uInt256Equal(this.hash, other.hash));
  this.serializeWire = (0, _Serializable.createSerializeWire)(this.serializeWireBase.bind(this));
  this.serializeUnsigned = (0, _Serializable.createSerializeWire)(this.serializeUnsignedBase.bind(this));
  this._networkFee = _utils2.default.lazyAsync((() => {
    var _ref11 = (0, _asyncToGenerator3.default)(function* (context) {
      const { getOutput, utilityToken } = context;

      const outputsForInputs = yield _promise2.default.all(_this3.inputs.map(function (input) {
        return getOutput(input);
      }));
      const inputValue = getUtilityValue({
        outputs: outputsForInputs,
        utilityToken
      });
      const outputValue = getUtilityValue({ outputs: _this3.outputs, utilityToken });
      return inputValue.sub(outputValue).sub(_this3.getSystemFee(context));
    });

    return function (_x11) {
      return _ref11.apply(this, arguments);
    };
  })());
  this.__getReferences = _utils2.default.lazyAsync((() => {
    var _ref12 = (0, _asyncToGenerator3.default)(function* ({
      getOutput
    }) {
      return _promise2.default.all(_this3.inputs.map(function (input) {
        return getOutput(input);
      }));
    });

    return function (_x12) {
      return _ref12.apply(this, arguments);
    };
  })());
  this.__getTransactionResults = _utils2.default.lazyAsync((() => {
    var _ref13 = (0, _asyncToGenerator3.default)(function* ({
      getOutput
    }) {
      const inputOutputs = yield _this3.getReferences({ getOutput });
      const results = {};
      const addOutputs = function (outputs, negative) {
        for (const output of outputs) {
          const key = _common2.default.uInt256ToHex(output.asset);
          if (results[key] == null) {
            results[key] = _utils2.default.ZERO;
          }

          results[key] = results[key].add(negative ? output.value.neg() : output.value);
        }
      };
      addOutputs(inputOutputs);
      addOutputs(_this3.outputs, true);

      return _lodash2.default.pickBy(results, function (value) {
        return !value.eq(_utils2.default.ZERO);
      });
    });

    return function (_x13) {
      return _ref13.apply(this, arguments);
    };
  })());
  this.__baseGetScriptHashesForVerifying = _utils2.default.lazyAsync((() => {
    var _ref14 = (0, _asyncToGenerator3.default)(function* ({
      getOutput,
      getAsset
    }) {
      const [inputHashes, outputHashes] = yield _promise2.default.all([_promise2.default.all(_this3.inputs.map((() => {
        var _ref15 = (0, _asyncToGenerator3.default)(function* (input) {
          const output = yield getOutput(input);
          return _common2.default.uInt160ToHex(output.address);
        });

        return function (_x15) {
          return _ref15.apply(this, arguments);
        };
      })())), _promise2.default.all(_this3.outputs.map((() => {
        var _ref16 = (0, _asyncToGenerator3.default)(function* (output) {
          const asset = yield getAsset({ hash: output.asset });
          if ((0, _AssetType.hasFlag)(asset.type, _AssetType.ASSET_TYPE.DUTY_FLAG)) {
            return _common2.default.uInt160ToHex(output.address);
          }

          return null;
        });

        return function (_x16) {
          return _ref16.apply(this, arguments);
        };
      })())).then(function (hashes) {
        return hashes.filter(Boolean);
      })]);
      const attributeHashes = _this3.attributes.map(function (attribute) {
        return attribute instanceof _attribute.UInt160Attribute && attribute.usage === _attribute.ATTRIBUTE_USAGE.SCRIPT ? _common2.default.uInt160ToHex(attribute.value) : null;
      }).filter(Boolean);

      return new _set2.default([...inputHashes, ...outputHashes, ...attributeHashes]);
    });

    return function (_x14) {
      return _ref14.apply(this, arguments);
    };
  })());
};

exports.default = TransactionBase;