'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _maxSafeInteger = require('babel-runtime/core-js/number/max-safe-integer');

var _maxSafeInteger2 = _interopRequireDefault(_maxSafeInteger);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _errors = require('../errors');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let BinaryReader = function () {
  function BinaryReader(buffer, index) {
    (0, _classCallCheck3.default)(this, BinaryReader);

    this.buffer = buffer;
    this.index = index || 0;
  }

  (0, _createClass3.default)(BinaryReader, [{
    key: 'hasMore',
    value: function hasMore() {
      return this.index < this.buffer.byteLength;
    }
  }, {
    key: 'clone',
    value: function clone() {
      return new BinaryReader(this.buffer, this.index);
    }
  }, {
    key: 'readBytes',
    value: function readBytes(numBytes) {
      const result = this.buffer.slice(this.index, this.index + numBytes);
      this.index += numBytes;
      return result;
    }
  }, {
    key: 'readInt8',
    value: function readInt8() {
      const result = this.buffer.readInt8(this.index);
      this.index += 1;
      return result;
    }
  }, {
    key: 'readUInt8',
    value: function readUInt8() {
      const result = this.buffer.readUInt8(this.index);
      this.index += 1;
      return result;
    }
  }, {
    key: 'readBoolean',
    value: function readBoolean() {
      return this.readBytes(1)[0] !== 0;
    }
  }, {
    key: 'readInt16LE',
    value: function readInt16LE() {
      const result = this.buffer.readInt16LE(this.index);
      this.index += 2;
      return result;
    }
  }, {
    key: 'readUInt16LE',
    value: function readUInt16LE() {
      const result = this.buffer.readUInt16LE(this.index);
      this.index += 2;
      return result;
    }
  }, {
    key: 'readInt32LE',
    value: function readInt32LE() {
      const result = this.buffer.readInt32LE(this.index);
      this.index += 4;
      return result;
    }
  }, {
    key: 'readUInt32LE',
    value: function readUInt32LE() {
      const result = this.buffer.readUInt32LE(this.index);
      this.index += 4;
      return result;
    }
  }, {
    key: 'readUInt64LE',
    value: function readUInt64LE() {
      return new _bn2.default(this.readBytes(8), 'le');
    }
  }, {
    key: 'readInt64LE',
    value: function readInt64LE() {
      const buffer = this.readBytes(8);
      return new _bn2.default(buffer, 'le').fromTwos(buffer.length * 8);
    }

    // NEO specific

  }, {
    key: 'readUInt160',
    value: function readUInt160() {
      return _common2.default.bufferToUInt160(this.readBytes(_common2.default.UINT160_BUFFER_BYTES));
    }
  }, {
    key: 'readUInt256',
    value: function readUInt256() {
      return _common2.default.bufferToUInt256(this.readBytes(_common2.default.UINT256_BUFFER_BYTES));
    }
  }, {
    key: 'readFixed8',
    value: function readFixed8() {
      return this.readInt64LE();
    }
  }, {
    key: 'readFixedString',
    value: function readFixedString(length) {
      const values = _lodash2.default.takeWhile([...this.readBytes(length)], value => value !== 0);
      return Buffer.from(values).toString('utf8');
    }
  }, {
    key: 'readArray',
    value: function readArray(read, max = 0x10000000) {
      const count = this.readVarUIntLE(new _bn2.default(max)).toNumber();
      const result = [];
      for (let i = 0; i < count; i += 1) {
        result.push(read());
      }

      return result;
    }
  }, {
    key: 'readObject',
    value: function readObject(read, max = 0x10000000) {
      const count = this.readVarUIntLE(new _bn2.default(max)).toNumber();
      const result = {};
      for (let i = 0; i < count; i += 1) {
        const { key, value } = read();
        result[key] = value;
      }

      return result;
    }
  }, {
    key: 'readVarBytesLE',
    value: function readVarBytesLE(max = _maxSafeInteger2.default) {
      return this.readBytes(this.readVarUIntLE(new _bn2.default(max)).toNumber());
    }
  }, {
    key: 'readVarUIntLE',
    value: function readVarUIntLE(max = new _bn2.default('18446744073709551615', 10)) {
      const fb = this.readUInt8();
      let value = new _bn2.default(fb);
      if (fb === 0xFD) {
        value = new _bn2.default(this.readUInt16LE());
      } else if (fb === 0xFE) {
        value = new _bn2.default(this.readUInt32LE());
      } else if (fb === 0xFF) {
        value = this.readUInt64LE();
      }

      if (value.gt(max)) {
        throw new _errors.InvalidFormatError();
      }

      return value;
    }
  }, {
    key: 'readVarString',
    value: function readVarString(max = 0x7fffffc7) {
      return this.readVarBytesLE(max).toString('utf8');
    }
  }, {
    key: 'readECPoint',
    value: function readECPoint() {
      const firstByte = this.readBytes(1);
      if (firstByte[0] === _common2.default.ECPOINT_INFINITY_BYTE) {
        return _common2.default.ECPOINT_INFINITY;
      }

      return _common2.default.bufferToECPoint(Buffer.concat([firstByte, this.readBytes(_common2.default.ECPOINT_BUFFER_BYTES - 1)]));
    }
  }, {
    key: 'remaining',
    get: function () {
      return this.buffer.length - this.index;
    }
  }, {
    key: 'remainingBuffer',
    get: function () {
      return this.buffer.slice(this.index);
    }
  }]);
  return BinaryReader;
}();

exports.default = BinaryReader;