'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _errors = require('../errors');

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let BinaryWriter = function () {
  function BinaryWriter() {
    (0, _classCallCheck3.default)(this, BinaryWriter);

    this.buffer = Buffer.from([]);
  }

  (0, _createClass3.default)(BinaryWriter, [{
    key: 'toBuffer',
    value: function toBuffer() {
      return this.buffer;
    }
  }, {
    key: 'writeBytes',
    value: function writeBytes(value) {
      this.buffer = Buffer.concat([this.buffer, value]);
      return this;
    }
  }, {
    key: 'writeUInt8',
    value: function writeUInt8(value) {
      const buffer = Buffer.allocUnsafe(1);
      buffer.writeUInt8(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: 'writeInt16LE',
    value: function writeInt16LE(value) {
      const buffer = Buffer.allocUnsafe(2);
      buffer.writeInt16LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: 'writeUInt16LE',
    value: function writeUInt16LE(value) {
      const buffer = Buffer.allocUnsafe(2);
      buffer.writeUInt16LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: 'writeInt32LE',
    value: function writeInt32LE(value) {
      const buffer = Buffer.allocUnsafe(4);
      buffer.writeInt32LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: 'writeUInt32LE',
    value: function writeUInt32LE(value) {
      const buffer = Buffer.allocUnsafe(4);
      buffer.writeUInt32LE(value, 0);
      return this.writeBytes(buffer);
    }
  }, {
    key: 'writeInt64LE',
    value: function writeInt64LE(value) {
      return this.writeBytes(value.toTwos(8 * 8).toArrayLike(Buffer, 'le', 8));
    }
  }, {
    key: 'writeUInt64LE',
    value: function writeUInt64LE(value) {
      return this.writeBytes(value.toArrayLike(Buffer, 'le', 8));
    }
  }, {
    key: 'writeBoolean',
    value: function writeBoolean(value) {
      this.writeBytes(Buffer.from([value ? 1 : 0]));
      return this;
    }

    // Special methods that don't fit the LE mold and/or are specific to NEO.

  }, {
    key: 'writeUInt160',
    value: function writeUInt160(hash) {
      return this.writeBytes(_common2.default.uInt160ToBuffer(hash));
    }
  }, {
    key: 'writeUInt256',
    value: function writeUInt256(hash) {
      return this.writeBytes(_common2.default.uInt256ToBuffer(hash));
    }
  }, {
    key: 'writeFixed8',
    value: function writeFixed8(value) {
      return this.writeInt64LE(value);
    }
  }, {
    key: 'writeFixedString',
    value: function writeFixedString(value, length) {
      if (value.length > length) {
        throw new _errors.InvalidFormatError();
      }

      const buffer = Buffer.from(value, 'utf8');
      if (buffer.length > length) {
        throw new _errors.InvalidFormatError();
      }

      this.writeBytes(buffer);
      if (buffer.length < length) {
        this.writeBytes(Buffer.alloc(length - buffer.length, 0));
      }

      return this;
    }
  }, {
    key: 'writeArray',
    value: function writeArray(values, write) {
      this.writeVarUIntLE(values.length);
      values.forEach(value => write(value));
      return this;
    }
  }, {
    key: 'writeObject',
    value: function writeObject(value, write) {
      const entries = _utils2.default.entries(value);
      this.writeVarUIntLE(entries.length);
      for (const [key, val] of entries) {
        write(key, val);
      }
      return this;
    }
  }, {
    key: 'writeVarBytesLE',
    value: function writeVarBytesLE(value) {
      this.writeVarUIntLE(value.length);
      return this.writeBytes(value);
    }
  }, {
    key: 'writeVarUIntLE',
    value: function writeVarUIntLE(valueIn) {
      const value = new _bn2.default(valueIn);
      if (value.lt(_utils2.default.ZERO)) {
        throw new _errors.InvalidFormatError();
      }

      if (value.lt(_utils2.default.FD)) {
        this.writeUInt8(value.toNumber());
      } else if (value.lte(_utils2.default.FFFF)) {
        this.writeUInt8(0xFD);
        this.writeUInt16LE(value.toNumber());
      } else if (value.lte(_utils2.default.FFFFFFFF)) {
        this.writeUInt8(0xFE);
        this.writeUInt32LE(value.toNumber());
      } else {
        this.writeUInt8(0xFF);
        this.writeUInt64LE(value);
      }

      return this;
    }
  }, {
    key: 'writeVarString',
    value: function writeVarString(value, max) {
      let buffer = Buffer.from(value, 'utf8');
      if (max != null) {
        buffer = buffer.slice(0, max);
      }
      return this.writeVarBytesLE(buffer);
    }
  }, {
    key: 'writeECPoint',
    value: function writeECPoint(value) {
      if (_common2.default.ecPointIsInfinity(value)) {
        return this.writeBytes(Buffer.from([_common2.default.ECPOINT_INFINITY_BYTE]));
      }

      return this.writeBytes(_common2.default.ecPointToBuffer(value));
    }
  }]);
  return BinaryWriter;
}();

exports.default = BinaryWriter;