'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _errors = require('../errors');

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const sizeOfUInt8 = 1;
const sizeOfBoolean = 1;
const sizeOfUInt16LE = 2;
const sizeOfUInt32LE = 4;
const sizeOfUInt64LE = 8;

const sizeOfUInt160 = _common2.default.UINT160_BUFFER_BYTES;
const sizeOfUInt256 = _common2.default.UINT256_BUFFER_BYTES;
const sizeOfECPoint = ecPoint => {
  if (_common2.default.ecPointIsInfinity(ecPoint)) {
    return 1;
  }

  return _common2.default.ECPOINT_BUFFER_BYTES;
};
const sizeOfFixed8 = 8;

const sizeOfVarUIntLE = valueIn => {
  const value = new _bn2.default(valueIn);
  if (value.lt(_utils2.default.ZERO)) {
    throw new _errors.InvalidFormatError();
  }

  if (value.lt(_utils2.default.FD)) {
    return sizeOfUInt8;
  } else if (value.lte(_utils2.default.FFFF)) {
    return sizeOfUInt8 + sizeOfUInt16LE;
  } else if (value.lte(_utils2.default.FFFFFFFF)) {
    return sizeOfUInt8 + sizeOfUInt32LE;
  }

  return sizeOfUInt8 + sizeOfUInt64LE;
};

const sizeOfVarBytesLE = value => sizeOfVarUIntLE(value.length) + value.length;

const sizeOfVarString = value => sizeOfVarBytesLE(Buffer.from(value, 'utf8'));

const sizeOfFixedString = length => length;

function sizeOfArray(values, sizeOf) {
  return values.reduce((acc, value) => acc + sizeOf(value), sizeOfVarUIntLE(values.length));
}

function sizeOfObject(obj, sizeOf) {
  const entries = _utils2.default.entries(obj);
  return entries.reduce((acc, [key, value]) => acc + sizeOf(key, value), entries.length);
};

exports.default = {
  sizeOfUInt8,
  sizeOfBoolean,
  sizeOfUInt16LE,
  sizeOfUInt32LE,
  sizeOfUInt64LE,
  sizeOfVarBytesLE,
  sizeOfVarString,
  sizeOfFixedString,
  sizeOfArray,
  sizeOfObject,
  sizeOfUInt160,
  sizeOfUInt256,
  sizeOfECPoint,
  sizeOfFixed8
};