'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.InvalidParamError = exports.UnknownOpError = undefined;

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _vm = require('../vm');

var _common = require('../common');

var _common2 = _interopRequireDefault(_common);

var _utils = require('./utils');

var _utils2 = _interopRequireDefault(_utils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let UnknownOpError = exports.UnknownOpError = function (_Error) {
  (0, _inherits3.default)(UnknownOpError, _Error);

  function UnknownOpError(byteCode) {
    (0, _classCallCheck3.default)(this, UnknownOpError);

    var _this = (0, _possibleConstructorReturn3.default)(this, (UnknownOpError.__proto__ || (0, _getPrototypeOf2.default)(UnknownOpError)).call(this, `Unknown op: ${byteCode}`));

    _this.byteCode = byteCode;
    return _this;
  }

  return UnknownOpError;
}(Error);

let InvalidParamError = exports.InvalidParamError = function (_Error2) {
  (0, _inherits3.default)(InvalidParamError, _Error2);

  function InvalidParamError() {
    (0, _classCallCheck3.default)(this, InvalidParamError);
    return (0, _possibleConstructorReturn3.default)(this, (InvalidParamError.__proto__ || (0, _getPrototypeOf2.default)(InvalidParamError)).call(this, 'Invalid Param'));
  }

  return InvalidParamError;
}(Error);

const BYTECODE_TO_BYTECODE_BUFFER = _lodash2.default.fromPairs((0, _values2.default)(_vm.OPCODE_TO_BYTECODE).map(byteCode => [byteCode, Buffer.from([byteCode])]));

let ScriptBuilder = function () {
  function ScriptBuilder() {
    (0, _classCallCheck3.default)(this, ScriptBuilder);

    this.buffers = [];
  }

  (0, _createClass3.default)(ScriptBuilder, [{
    key: 'emitPush',
    value: function emitPush(value) {
      if (value.length <= _vm.OPCODE_TO_BYTECODE.PUSHBYTES75) {
        this.emitOpByte(value.length, value);
      } else if (value.length < 0x100) {
        this.emitOp('PUSHDATA1');
        this.emitUInt8(value.length);
        this.emit(value);
      } else if (value.length < 0x10000) {
        this.emitOp('PUSHDATA2');
        this.emitUInt16LE(value.length);
        this.emit(value);
        // TODO: Check this condition if (data.Length < 0x100000000L)
      } else {
        this.emitOp('PUSHDATA4');
        this.emitUInt32LE(value.length);
        this.emit(value);
      }

      return this;
    }
  }, {
    key: 'emitUInt8',
    value: function emitUInt8(value) {
      const buff = Buffer.allocUnsafe(1);
      buff.writeUInt8(value, 0);
      return this.emit(buff);
    }
  }, {
    key: 'emitUInt16LE',
    value: function emitUInt16LE(value) {
      const buff = Buffer.allocUnsafe(2);
      buff.writeUInt16LE(value, 0);
      return this.emit(buff);
    }
  }, {
    key: 'emitUInt32LE',
    value: function emitUInt32LE(value) {
      const buff = Buffer.allocUnsafe(4);
      buff.writeUInt32LE(value, 0);
      return this.emit(buff);
    }
  }, {
    key: 'emitPushInt',
    value: function emitPushInt(valueIn) {
      const value = new _bn2.default(valueIn);
      if (value.eq(_utils2.default.NEGATIVE_ONE)) {
        return this.emitOp('PUSHM1');
      } else if (value.eq(_utils2.default.ZERO)) {
        return this.emitOp('PUSH0');
      } else if (value.gt(_utils2.default.ZERO) && value.lt(_utils2.default.SIXTEEN)) {
        return this.emitOpByte(_vm.OPCODE_TO_BYTECODE.PUSH1 - 1 + value.toNumber());
      }

      return this.emitPush(_utils2.default.toSignedBuffer(value));
    }
  }, {
    key: 'emitPushUInt160',
    value: function emitPushUInt160(value) {
      return this.emitPush(_common2.default.uInt160ToBuffer(value));
    }
  }, {
    key: 'emitPushECPoint',
    value: function emitPushECPoint(ecPoint) {
      return this.emitPush(_common2.default.ecPointToBuffer(ecPoint));
    }
  }, {
    key: 'emitPushString',
    value: function emitPushString(value) {
      return this.emitPush(Buffer.from(value, 'utf8'));
    }
  }, {
    key: 'emitOp',
    value: function emitOp(op, buffer) {
      this.emitOpByte(_vm.OPCODE_TO_BYTECODE[op], buffer);
      return this;
    }
  }, {
    key: 'emitPushParam',
    value: function emitPushParam(param) {
      if (_common2.default.isUInt160(param)) {
        return this.emitPushUInt160(_common2.default.asUInt160(param));
      } else if (typeof param === 'number' || param instanceof _bn2.default) {
        return this.emitPushInt(param);
      } else if (typeof param === 'string') {
        return this.emitPushString(param);
      }

      throw new InvalidParamError();
    }
  }, {
    key: 'emitAppCall',
    value: function emitAppCall(scriptHash, operation, ...params) {
      for (let i = params.length - 1; i >= 0; i -= 1) {
        this.emitPushParam(params[i]);
      }
      this.emitPushParam(params.length);
      this.emitOp('PACK');
      this.emitPushParam(operation);
      return this.emitOp('APPCALL', _common2.default.uInt160ToBuffer(scriptHash));
    }
  }, {
    key: 'emitOpByte',
    value: function emitOpByte(byteCodeIn, buffer) {
      const byteCode = `${byteCodeIn == null ? '' : byteCodeIn}`;
      const byteCodeBuffer = BYTECODE_TO_BYTECODE_BUFFER[byteCode];
      if (byteCodeBuffer == null) {
        throw new UnknownOpError(byteCode);
      }
      this.emit(byteCodeBuffer);
      this.emit(buffer);
      return this;
    }
  }, {
    key: 'emit',
    value: function emit(buffer) {
      if (buffer != null) {
        this.buffers.push(buffer);
      }
      return this;
    }
  }, {
    key: 'build',
    value: function build() {
      return Buffer.concat(this.buffers);
    }
  }]);
  return ScriptBuilder;
}();

exports.default = ScriptBuilder;