'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _values = require('babel-runtime/core-js/object/values');

var _values2 = _interopRequireDefault(_values);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _bn = require('bn.js');

var _bn2 = _interopRequireDefault(_bn);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const USHORT_MAX_NUMBER = 65535;
const USHORT_MAX_NUMBER_PLUS_ONE = 65535 + 1;
const USHORT_MAX = new _bn2.default(USHORT_MAX_NUMBER);
const USHORT_MAX_PLUS_ONE = USHORT_MAX.addn(1);
const UINT_MAX_NUMBER = 4294967295;
const UINT_MAX = new _bn2.default(UINT_MAX_NUMBER);
const ZERO = new _bn2.default(0);
const ONE = new _bn2.default(1);
const NEGATIVE_ONE = new _bn2.default(-1);
const ONE_HUNDRED_MILLION = new _bn2.default(100000000);

const toSignedBuffer = value => {
  const buff = value.toArrayLike(Buffer, 'le');
  if (value.isNeg()) {
    return buff;
  }

  return value.toArrayLike(Buffer, 'le', buff.length + 1);
};

const fromSignedBuffer = value => value.length === 0 ? ZERO : new _bn2.default(value, 'le').fromTwos(value.length * 8);

// TODO: This might be incorrect with the shenanigans we're doing above
const not = value => value.notn(value.bitLength());

const getBoolean = value => value.some(byte => byte !== 0);

const booleanToBuffer = value => Buffer.from([value ? 1 : 0]);

const toASCII = bytes => {
  let result = '';
  for (let i = 0; i < bytes.length; i += 1) {
    result += String.fromCharCode(bytes.readUInt8(i));
  }

  return result;
};

const toUTF8 = bytes => bytes.toString('utf8');

function keys(obj) {
  return (0, _keys2.default)(obj);
}

function values(obj) {
  return (0, _values2.default)(obj);
}

function entries(obj) {
  return (0, _entries2.default)(obj);
}

const reverse = src => {
  const out = Buffer.allocUnsafe(src.length);
  for (let i = 0, j = src.length - 1; i <= j; i += 1, j -= 1) {
    out[i] = src[j];
    out[j] = src[i];
  }

  return out;
};

const calculateClaimAmount = (() => {
  var _ref = (0, _asyncToGenerator3.default)(function* ({
    coins,
    decrementInterval,
    generationAmount,
    getSystemFee
  }) {
    const grouped = values(_lodash2.default.groupBy(coins, function (coin) {
      return `${coin.startHeight}:${coin.endHeight}`;
    }));
    const claimed = yield _promise2.default.all(grouped.map((() => {
      var _ref2 = (0, _asyncToGenerator3.default)(function* (coinsGroup) {
        const { startHeight, endHeight } = coinsGroup[0];

        let amount = ZERO;
        let ustart = Math.floor(startHeight / decrementInterval);
        if (ustart < generationAmount.length) {
          let istart = startHeight % decrementInterval;
          let uend = Math.floor(endHeight / decrementInterval);
          let iend = endHeight % decrementInterval;
          if (uend >= generationAmount.length) {
            uend = generationAmount.length;
            iend = 0;
          }

          if (iend === 0) {
            uend -= 1;
            iend = decrementInterval;
          }

          while (ustart < uend) {
            amount = amount.addn((decrementInterval - istart) * generationAmount[ustart]);
            ustart += 1;
            istart = 0;
          }

          amount = amount.addn((iend - istart) * generationAmount[ustart]);
        }

        const [sysFeeEnd, sysFeeStart] = yield _promise2.default.all([getSystemFee(endHeight - 1), startHeight === 0 ? _promise2.default.resolve(ZERO) : getSystemFee(startHeight - 1)]);

        amount = amount.add(sysFeeEnd.sub(sysFeeStart).div(ONE_HUNDRED_MILLION));
        const totalValue = coinsGroup.reduce(function (acc, { value }) {
          return acc.add(value);
        }, ZERO);
        return [totalValue, amount];
      });

      return function (_x2) {
        return _ref2.apply(this, arguments);
      };
    })()));

    return claimed.reduce(function (acc, [value, amount]) {
      return acc.add(value.div(ONE_HUNDRED_MILLION).mul(amount));
    }, ZERO);
  });

  return function calculateClaimAmount(_x) {
    return _ref.apply(this, arguments);
  };
})();

const randomUInt = () => Math.floor(Math.random() * UINT_MAX_NUMBER);

exports.default = {
  FD: new _bn2.default(0xFD),
  FFFF: new _bn2.default(0xFFFF),
  FFFFFFFF: new _bn2.default(0xFFFFFFFF),
  ZERO,
  ONE,
  NEGATIVE_ONE,
  INT_MAX_VALUE: new _bn2.default(2147483647),
  SATOSHI: ONE,
  NEGATIVE_SATOSHI: NEGATIVE_ONE,
  USHORT_MAX_NUMBER,
  USHORT_MAX_NUMBER_PLUS_ONE,
  USHORT_MAX,
  USHORT_MAX_PLUS_ONE,
  UINT_MAX_NUMBER,
  UINT_MAX,
  ONE_HUNDRED_MILLION,
  EIGHT: new _bn2.default(8),
  TEN: new _bn2.default(10),
  SIXTEEN: new _bn2.default(16),
  toSignedBuffer,
  fromSignedBuffer,
  not,
  getBoolean,
  booleanToBuffer,
  toASCII,
  toUTF8,
  keys,
  values,
  entries,
  reverse,
  calculateClaimAmount,
  randomUInt
};